#!/usr/bin/env python

"""
_DQMHarvest_t_
"""
from __future__ import print_function

import os
import threading
import unittest

from Utils.PythonVersion import PY3

from WMCore.DAOFactory import DAOFactory
from WMCore.Database.CMSCouch import CouchServer, Document
from WMCore.WMSpec.StdSpecs.DQMHarvest import DQMHarvestWorkloadFactory
from WMCore.WMSpec.WMSpecErrors import WMSpecFactoryException
from WMCore.WorkQueue.WMBSHelper import WMBSHelper
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMQuality.TestInitCouchApp import TestInitCouchApp

REQUEST = {
    "AcquisitionEra": "Run2016F",
    "CMSSWVersion": "CMSSW_8_0_20",
    "Campaign": "Campaign-OVERRIDE-ME",
    "Comments": "Harvest all 37 runs in byRun mode (separate jobs)",
    "CouchURL": os.environ["COUCHURL"],
    "ConfigCacheUrl": os.environ["COUCHURL"],
    "CouchDBName": "dqmharvest_t",
    "DQMConfigCacheID": "253c586d672c6c7a88c048d8c7b62135",
    "DQMHarvestUnit": "byRun",
    "DQMUploadUrl": "https://cmsweb-testbed.cern.ch/dqm/dev",
    "DbsUrl": "https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader",
    "GlobalTag": "80X_dataRun2_2016SeptRepro_v3",
    "InputDataset": "/NoBPTX/Run2016F-23Sep2016-v1/DQMIO",
    "Memory": 1000,
    "Multicore": 1,
    "PrepID": "TEST-Harvest-ReReco-Run2016F-v1-NoBPTX-23Sep2016-0001",
    "ProcessingString": "23Sep2016",
    "ProcessingVersion": 1,
    "RequestPriority": 999999,
    "RequestString": "RequestString-OVERRIDE-ME",
    "RequestType": "DQMHarvest",
    "Requestor": "amaltaro",
    "ScramArch": "slc6_amd64_gcc530",
    "SizePerEvent": 1600,
    "TimePerEvent": 1
}


class DQMHarvestTests(EmulatedUnitTestCase):
    """
    _DQMHarvestTests_

    Tests the DQMHarvest spec file
    """

    def setUp(self):
        """
        _setUp_

        Initialize the database and couch.
        """
        super(DQMHarvestTests, self).setUp()
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setupCouch("dqmharvest_t", "ConfigCache")
        self.testInit.setSchema(customModules=["WMCore.WMBS"], useDefault=False)

        couchServer = CouchServer(os.environ["COUCHURL"])
        self.configDatabase = couchServer.connectDatabase("dqmharvest_t")
        self.testInit.generateWorkDir()

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)
        self.listTasksByWorkflow = self.daoFactory(classname="Workflow.LoadFromName")
        self.listFilesets = self.daoFactory(classname="Fileset.List")
        self.listSubsMapping = self.daoFactory(classname="Subscriptions.ListSubsAndFilesetsFromWorkflow")
        if PY3:
            self.assertItemsEqual = self.assertCountEqual
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out the database.
        """
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        self.testInit.delWorkDir()
        super(DQMHarvestTests, self).tearDown()
        return

    def injectDQMHarvestConfig(self):
        """
        _injectDQMHarvest_

        Create a bogus config cache document for DQMHarvest and
        inject it into couch.  Return the ID of the document.
        """
        newConfig = Document()
        newConfig["info"] = None
        newConfig["config"] = None
        newConfig["md5hash"] = "eb1c38cf50e14cf9fc31278a5c8e234f"
        newConfig["pset_hash"] = "7c856ad35f9f544839d8525ca10876a7"
        newConfig["owner"] = {"group": "DATAOPS", "user": "amaltaro"}
        newConfig["pset_tweak_details"] = {"process": {"outputModules_": []}}
        result = self.configDatabase.commitOne(newConfig)
        return result[0]["id"]

    def testDQMHarvest(self):
        """
        Build a DQMHarvest workload
        """
        testArguments = DQMHarvestWorkloadFactory.getTestArguments()
        testArguments.update(REQUEST)
        testArguments.update({
            "DQMConfigCacheID": self.injectDQMHarvestConfig(),
            "LumiList": {"251643": [[1, 15], [50, 70]], "251721": [[50, 100], [110, 120]]}
        })
        testArguments.pop("ConfigCacheID", None)

        factory = DQMHarvestWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        # test workload properties
        self.assertEqual(testWorkload.getDashboardActivity(), "harvesting")
        self.assertEqual(testWorkload.getCampaign(), "Campaign-OVERRIDE-ME")
        self.assertEqual(testWorkload.getAcquisitionEra(), "Run2016F")
        self.assertEqual(testWorkload.getProcessingString(), "23Sep2016")
        self.assertEqual(testWorkload.getProcessingVersion(), 1)
        self.assertEqual(testWorkload.getPrepID(), "TEST-Harvest-ReReco-Run2016F-v1-NoBPTX-23Sep2016-0001")
        self.assertEqual(testWorkload.getCMSSWVersions(), ['CMSSW_8_0_20'])
        self.assertEqual(sorted(testWorkload.getLumiList().keys()), ['251643', '251721'])
        self.assertEqual(sorted(testWorkload.getLumiList().values()), [[[1, 15], [50, 70]], [[50, 100], [110, 120]]])
        self.assertEqual(testWorkload.data.policies.start.policyName, "Dataset")

        # test workload tasks and steps
        tasks = testWorkload.listAllTaskNames()
        self.assertEqual(len(tasks), 2)
        self.assertEqual(sorted(tasks), ['EndOfRunDQMHarvest', 'EndOfRunDQMHarvestLogCollect'])

        task = testWorkload.getTask(tasks[0])
        self.assertEqual(task.name(), "EndOfRunDQMHarvest")
        self.assertEqual(task.getPathName(), "/TestWorkload/EndOfRunDQMHarvest")
        self.assertEqual(task.taskType(), "Harvesting", "Wrong task type")
        self.assertEqual(task.jobSplittingAlgorithm(), "Harvest", "Wrong job splitting algo")
        self.assertFalse(task.getTrustSitelists().get('trustlists'), "Wrong input location flag")
        self.assertFalse(task.inputRunWhitelist())

        self.assertEqual(sorted(task.listAllStepNames()), ['cmsRun1', 'logArch1', 'upload1'])
        self.assertEqual(task.getStep("cmsRun1").stepType(), "CMSSW")
        self.assertEqual(task.getStep("logArch1").stepType(), "LogArchive")
        self.assertEqual(task.getStep("upload1").stepType(), "DQMUpload")

        return

    def testDQMHarvestFailed(self):
        """
        Build a DQMHarvest workload without a DQM config doc
        """
        testArguments = DQMHarvestWorkloadFactory.getTestArguments()
        testArguments.update(REQUEST)
        testArguments.update({
            "ConfigCacheID": self.injectDQMHarvestConfig()
        })
        testArguments.pop("DQMConfigCacheID", None)

        factory = DQMHarvestWorkloadFactory()
        with self.assertRaises(WMSpecFactoryException):
            factory.factoryWorkloadConstruction("TestBadWorkload", testArguments)
        return

    def testFilesets(self):
        """
        Test workflow tasks, filesets and subscriptions creation
        """
        # expected tasks, filesets, subscriptions, etc
        expOutTasks = []
        expWfTasks = ['/TestWorkload/EndOfRunDQMHarvest',
                      '/TestWorkload/EndOfRunDQMHarvest/EndOfRunDQMHarvestLogCollect']
        expFsets = ['TestWorkload-EndOfRunDQMHarvest-/NoBPTX/Run2016F-23Sep2016-v1/DQMIO',
                    '/TestWorkload/EndOfRunDQMHarvest/unmerged-logArchive']

        subMaps = [(2, '/TestWorkload/EndOfRunDQMHarvest/unmerged-logArchive',
                    '/TestWorkload/EndOfRunDQMHarvest/EndOfRunDQMHarvestLogCollect', 'MinFileBased', 'LogCollect'),
                   (1, 'TestWorkload-EndOfRunDQMHarvest-/NoBPTX/Run2016F-23Sep2016-v1/DQMIO',
                    '/TestWorkload/EndOfRunDQMHarvest', 'Harvest', 'Harvesting')]

        testArguments = DQMHarvestWorkloadFactory.getTestArguments()
        testArguments.update(REQUEST)
        testArguments['DQMConfigCacheID'] = self.injectDQMHarvestConfig()
        testArguments.pop("ConfigCacheID", None)

        factory = DQMHarvestWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        testWMBSHelper = WMBSHelper(testWorkload, "EndOfRunDQMHarvest",
                                    blockName=testArguments['InputDataset'],
                                    cachepath=self.testInit.testDir)
        testWMBSHelper.createTopLevelFileset()
        testWMBSHelper._createSubscriptionsInWMBS(testWMBSHelper.topLevelTask, testWMBSHelper.topLevelFileset)

        self.assertItemsEqual(testWorkload.listOutputProducingTasks(), expOutTasks)

        workflows = self.listTasksByWorkflow.execute(workflow="TestWorkload")
        self.assertItemsEqual([item['task'] for item in workflows], expWfTasks)

        # returns a tuple of id, name, open and last_update
        filesets = self.listFilesets.execute()
        self.assertItemsEqual([item[1] for item in filesets], expFsets)

        subscriptions = self.listSubsMapping.execute(workflow="TestWorkload", returnTuple=True)
        self.assertItemsEqual(subscriptions, subMaps)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""
WorkQueuManager test
"""
from __future__ import print_function, division

from builtins import next
import unittest

from WMCore_t.WorkQueue_t.WorkQueueTestCase import WorkQueueTestCase

from WMCore.WorkQueue.WorkQueue import globalQueue


def getFirstTask(wmspec):
    """Return the 1st top level task"""
    return next(wmspec.taskIterator())


class WorkQueueManagerTest(WorkQueueTestCase):
    """
    TestCase for WorkQueueManagerTest module
    """

    _maxMessage = 10

    def setSchema(self):
        self.schema = []
        self.couchApps = ["WorkQueue"]

    def getConfig(self):
        """
        _createConfig_

        General config file
        """
        # configPath=os.path.join(WMCore.WMInit.getWMBASE(), \
        #                        'src/python/WMComponent/WorkQueueManager/DefaultConfig.py')):


        config = self.testInit.getConfiguration()
        # http://www.logilab.org/ticket/8961
        # pylint: disable=E1101, E1103
        config.component_("WorkQueueManager")
        config.section_("General")
        config.General.workDir = "."
        config.WorkQueueManager.team = 'team_usa'
        config.WorkQueueManager.requestMgrHost = 'cmssrv49.fnal.gov:8585'
        config.WorkQueueManager.serviceUrl = "http://cmssrv18.fnal.gov:6660"

        config.WorkQueueManager.logLevel = 'INFO'
        config.WorkQueueManager.pollInterval = 10
        config.WorkQueueManager.level = "GlobalQueue"

        return config

    def setupGlobalWorkqueue(self):
        """Return a workqueue instance"""

        globalQ = globalQueue(CacheDir=self.workDir,
                              QueueURL='global.example.com',
                              Teams=["The A-Team", "some other bloke"],
                              DbName='workqueue_t_global')
        return globalQ

    def testComponentBasic(self):
        """
        Tests the components, as in sees if they load.
        Otherwise does nothing.
        """
        return
        # TODO: What used to be here, stopping pylint from complaining about unreachable code
        # myThread = threading.currentThread()
        #
        # config = self.getConfig()
        #
        # testWorkQueueManager = WorkQueueManager(config)
        # testWorkQueueManager.prepareToStart()
        #
        # time.sleep(30)
        # print("Killing")
        # myThread.workerThreadManager.terminateWorkers()
        #
        # return


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""
Unit testing base class that turns on emulators
"""

from __future__ import (division, print_function)

import unittest

import mock

from WMQuality.Emulators.CRICClient.MockCRICApi import MockCRICApi
from WMQuality.Emulators.Cache.MockMemoryCacheStruct import MockMemoryCacheStruct
from WMQuality.Emulators.DBSClient.MockDbsApi import MockDbsApi
from WMQuality.Emulators.LogDB.MockLogDB import MockLogDB
from WMQuality.Emulators.PyCondorAPI.MockPyCondorAPI import MockPyCondorAPI
from WMQuality.Emulators.ReqMgrAux.MockReqMgrAux import MockReqMgrAux
from WMQuality.Emulators.RucioClient.MockRucioApi import MockRucioApi


class EmulatedUnitTestCase(unittest.TestCase):
    """
    Class that can be imported to switch to 'mock'ed versions of
    services.
    """

    def __init__(self, methodName='runTest', mockDBS=True,
                 mockReqMgrAux=True, mockLogDB=True,
                 mockMemoryCache=True, mockPyCondor=True,
                 mockCRIC=True, mockRucio=True):
        self.mockDBS = mockDBS
        self.mockReqMgrAux = mockReqMgrAux
        self.mockLogDB = mockLogDB
        self.mockMemoryCache = mockMemoryCache
        self.mockPyCondor = mockPyCondor
        self.mockCRIC = mockCRIC
        self.mockRucio = mockRucio
        super(EmulatedUnitTestCase, self).__init__(methodName)

    def setUp(self):
        """
        Start the various mocked versions and add cleanups in case of exceptions
        Note: patch has to be applied to where the object is loaded/imported, not
        to its origin location.

        TODO: parameters to turn off emulators individually
        """

        if self.mockDBS:
            self.dbsPatchers = []
            patchDBSAt = ["dbs.apis.dbsClient.DbsApi",
                          "WMCore.Services.DBS.DBS3Reader.DbsApi"]
            for module in patchDBSAt:
                self.dbsPatchers.append(mock.patch(module, new=MockDbsApi))
                self.dbsPatchers[-1].start()
                self.addCleanup(self.dbsPatchers[-1].stop)

        if self.mockRucio:
            self.rucioPatchers = []
            patchRucioAt = ['WMCore.WorkQueue.WorkQueue.Rucio',
                            'WMCore.WorkQueue.WorkQueueReqMgrInterface.Rucio',
                            'WMCore.WorkQueue.Policy.Start.StartPolicyInterface.Rucio',
                            'WMComponent.RucioInjector.RucioInjectorPoller.Rucio',
                            'WMCore.WMSpec.Steps.Fetchers.PileupFetcher.Rucio',
                            'WMCore_t.WMSpec_t.Steps_t.Fetchers_t.PileupFetcher_t.Rucio',
                            'WMCore_t.WorkQueue_t.WMBSHelper_t.Rucio']
            for module in patchRucioAt:
                self.rucioPatchers.append(mock.patch(module, new=MockRucioApi))
                self.rucioPatchers[-1].start()
                self.addCleanup(self.rucioPatchers[-1].stop)

        if self.mockReqMgrAux:
            self.reqMgrAuxPatchers = []
            patchReqMgrAuxAt = ['WMCore.Services.ReqMgrAux.ReqMgrAux.ReqMgrAux',
                                'WMComponent.JobSubmitter.JobSubmitterPoller.ReqMgrAux',
                                'WMComponent.ErrorHandler.ErrorHandlerPoller.ReqMgrAux']
            for module in patchReqMgrAuxAt:
                self.reqMgrAuxPatchers.append(mock.patch(module, new=MockReqMgrAux))
                self.reqMgrAuxPatchers[-1].start()
                self.addCleanup(self.reqMgrAuxPatchers[-1].stop)

        if self.mockLogDB:
            self.logDBPatcher = mock.patch('WMCore.Services.LogDB.LogDB.LogDB',
                                           new=MockLogDB)
            self.inUseLogDB = self.logDBPatcher.start()
            self.addCleanup(self.logDBPatcher.stop)

        if self.mockMemoryCache:
            self.memoryCachePatcher = mock.patch('WMCore.Cache.GenericDataCache.MemoryCacheStruct',
                                                 new=MockMemoryCacheStruct)
            self.inUseMemoryCache = self.memoryCachePatcher.start()
            self.addCleanup(self.memoryCachePatcher.stop)

        if self.mockPyCondor:
            self.condorPatcher = mock.patch('WMCore.Services.PyCondor.PyCondorAPI.PyCondorAPI',
                                            new=MockPyCondorAPI)
            self.condorPatcher.start()
            self.addCleanup(self.condorPatcher.stop)

        if self.mockCRIC:
            self.cricPatchers = []
            patchCRICAt = ['WMCore.ReqMgr.Tools.cms.CRIC',
                           'WMCore.WorkQueue.WorkQueue.CRIC',
                           'WMCore.WorkQueue.WorkQueueUtils.CRIC',
                           'WMCore.WorkQueue.Policy.Start.StartPolicyInterface.CRIC']
            for module in patchCRICAt:
                self.cricPatchers.append(mock.patch(module, new=MockCRICApi))
                self.cricPatchers[-1].start()
                self.addCleanup(self.cricPatchers[-1].stop)

        return



#!/usr/bin/env python
"""
_ListRunsWorkflow_

MySQL implementation of DBS3Buffer.ListRunsWorkflow
"""

from WMComponent.DBS3Buffer.Oracle.ListRunsWorkflow import ListRunsWorkflow as OracleListRunsWorkflow

class ListRunsWorkflow(OracleListRunsWorkflow):
    """
    _ListRunsWorkflow_

    Identical to Oracle version
    """



#!/usr/bin/env python
"""
_SetCouchID_

MySQL implementation of Jobs.SetCouchID
"""




from WMCore.Database.DBFormatter import DBFormatter

class SetCouchID(DBFormatter):
    """
    _SetCouchID_

    Update the id of the couch document for the given job.
    """
    sql = "UPDATE wmbs_job SET couch_record = :couchid WHERE id = :jobid"

    def execute(self, jobID = None, couchID = None, bulkList = None, conn = None,
                transaction = False):
        """
        _execute_

        Update the location of the couch record for the job.
        """

        if isinstance(bulkList, list):
            binds = bulkList
        else:
            binds = {"jobid": jobID, "couchid": couchID}

        self.dbi.processData(self.sql, binds,
                             conn = conn, transaction = transaction)
        return



#!/usr/bin/env python
"""
_TaskSpace_


Runtime utils for a Task


"""

from builtins import object
import os
import sys
import inspect
import pickle

from WMCore.WMSpec.WMWorkload import WMWorkloadHelper


def preloadWorkload(x):
    """
    _preloadWorkload_

    Method decorator to ensure that accessors to the workload are
    called only when the workload has been loaded

    Use only for decorating no-arg getters

    """
    def wrapper(self):
        self.getWMWorkload()
        return x(self)
    return wrapper

class TaskSpace(object):
    """
    _TaskSpace_

    Util container for runtime operations within a Task & its
    constituent steps

    """
    def __init__(self, **args):
        self.taskName = args['TaskName']
        self.location = os.path.dirname(inspect.getsourcefile(args['Locator']))
        self.task = None
        self.workload = None







    def getWMWorkload(self):
        """
        _getWMTask_

        Get the WMTask for this space

        TODO: Refactor to getWMWorkload method

        """
        if self.workload != None:
            return self.workload
        try:
            import WMSandbox
        except ImportError as ex:
            msg = "Error importing WMSandbox module"
            msg += str(ex)
            raise RuntimeError(msg)

        wmsandboxLoc = inspect.getsourcefile(WMSandbox)
        workloadPcl = wmsandboxLoc.replace("__init__.py","WMWorkload.pkl")

        with open(workloadPcl, 'rb') as handle:
            wmWorkload = pickle.load(handle)
        self.workload = WMWorkloadHelper(wmWorkload)
        return

    @preloadWorkload
    def workloadName(self):
        return self.workload.name()

    @preloadWorkload
    def getWMTask(self):
        """
        _getWMTask_

        Get the WMTask instance from the workload

        """
        return self.workload.getTaskByPath(self.taskName)


    def stepSpaces(self):
        """
        _stepSpaces_

        List step spaces available in this

        """
        result = []
        for item in os.listdir(self.location):
            location = os.path.join(self.location, item)
            if not os.path.isdir(location): continue
            initModule = os.path.join(location, "__init__.py")
            if not os.path.exists(initModule): continue
            #TODO: Test that it is really a StepSpace not some random py module
            result.append(item)
        return result

    def stepSpace(self, stepName):
        """
        _stepSpace_

        Load in the step Space with the name provided.
        Returns the step space instance from the directory

        """
        modName = "WMTaskSpace.%s" % stepName
        if modName in sys.modules:
            space = sys.modules[modName]
        else:
            try:
                #space = __import__(modName, globals(), locals(), ['stepSpace'], -1)
                space = __import__(modName, globals(), locals(), ['stepSpace'])

            except ImportError as ex:
                # TODO: Dedicated exception class
                msg = "Unable to import StepSpace from %s:\n" % modName
                msg += str(ex)
                raise RuntimeError(msg)


        stepSpace = getattr(space, "stepSpace", None)
        if stepSpace == None:
            # TODO: Dedicated Exception class
            msg = "No stepSpace Attribute in module %s" % modName
            raise RuntimeError(msg)

        setattr(stepSpace, "taskSpace", self)

        return stepSpace



#!/usr/bin/env python
"""
_GetCountByState_

MySQL implementation of GetCountByState
"""
from __future__ import division

from WMCore.Database.DBFormatter import DBFormatter


class GetCountByState(DBFormatter):
    """
    Retrieve the number of wmbs jobs in a given state

    Returns an integer value.
    """
    sql = """
          SELECT count(*) FROM wmbs_job
              WHERE state = (SELECT id FROM wmbs_job_state WHERE name = :state)
          """

    def execute(self, state, conn=None, transaction=False):
        result = self.dbi.processData(self.sql, {'state': state},
                                      conn=conn,
                                      transaction=transaction)
        return self.format(result)[0][0]



#!/usr/bin/env python
"""
_Job_t_

Testcase for the Job class.
"""

from builtins import range
import unittest, logging, random, time

from WMCore.DataStructs.Job import Job
from WMCore.DataStructs.Fileset import Fileset
from WMCore.DataStructs.File import File
from WMCore.DataStructs.Workflow import Workflow
from WMCore.DataStructs.Subscription import Subscription
from WMCore.DataStructs.Run import Run

from unittest import TestCase

class JobTest(unittest.TestCase):
    """
    _JobTest_

    Testcase for the Job class

    Instantiate a dummy Job object with a dummy Subscription
    and a dummy Fileset full of random files as input
    """

    def setUp(self):
        """
        _setUp_

        Initial Setup for the Job Testcase
        """
        self.inputFiles = []

        for i in range(1,1000):
            lfn = "/store/data/%s/%s/file.root" % (random.randint(1000, 9999),
                                                   random.randint(1000, 9999))
            size = random.randint(1000, 2000)
            events = 1000
            run = random.randint(0, 2000)
            lumi = random.randint(0, 8)

            file = File(lfn = lfn, size = size, events = events, checksums = {"cksum": "1"})
            file.addRun(Run(run, *[lumi]))
            self.inputFiles.append(file)

        self.dummyJob = Job(files = self.inputFiles)
        return

    def tearDown(self):
        """
        No tearDown method for this Testcase
        """
        pass

    def testGetFilesList(self):
        """
        _testGetFilesList_

        Verify that the Job::getFiles(type = "list") method returns the same
        files in the same order that they were passed in.
        """
        assert self.dummyJob.getFiles() == self.inputFiles, \
            "ERROR: Initial fileset does not match Job fileset"

        return

    def testGetFilesSet(self):
        """
        _testGetFilesSet_

        Verify that the Job::getFiles(type = "set") method returns the correct
        input files in the form of a set.
        """
        assert self.dummyJob.getFiles(type = "set") == set(self.inputFiles), \
            "ERROR: getFiles(type = 'set') does not work correctly."

        return

    def testGetFilesLFN(self):
        """
        _testGetFilesLFN_

        Verify that the Job::getFiles(type = "lfn") method returns the same
        files in the same order that they were passed in.
        """
        jobLFNs = self.dummyJob.getFiles(type = "lfn")

        goldenLFNs = []
        for file in self.inputFiles:
            goldenLFNs.append(file["lfn"])

        assert len(goldenLFNs) == len(jobLFNs), \
            "ERROR: Job has different number of files than input"

        for jobLFN in jobLFNs:
            assert jobLFN in goldenLFNs, \
                "ERROR: LFN missing from job."

        return

    def testAddFile(self):
        """
        _testAddFile_

        Verify that the Job::addFile() method works properly.
        """
        dummyFileAddFile = File("/tmp/dummyFileAddFileTest", 1234, 1, 2)
        self.dummyJob.addFile(dummyFileAddFile)

        assert dummyFileAddFile in self.dummyJob.getFiles(), \
            "ERROR: Couldn't add file to Job - addFile method error"

        return

    def testChangeState(self):
        """
        _testChangeState_

        Verify that the Job::changeState() method updates the state and the
        state time.
        """
        currentTime = time.time()
        self.dummyJob.changeState("created")

        assert self.dummyJob["state_time"] > currentTime - 1 and \
            self.dummyJob["state_time"] < currentTime + 1, \
            "ERROR: State time not updated on state change"

        assert self.dummyJob["state"] == "created", \
            "ERROR: Couldn't change Job state - changeState method error"

    def testChangeOutcome(self):
        """
        _testChangeOutcome_

        Verify that the Job::changeOutcome() method changes the final outcome
        of the job.
        """
        self.dummyJob.changeOutcome("success")

        assert self.dummyJob["outcome"] == "success", \
            "ERROR: Job outcome failed to update."

        return

    def testGetBaggage(self):
        """
        test that setting/accessing the Job Baggage ConfigSection works
        """
        self.dummyJob.addBaggageParameter("baggageContents", True)
        self.dummyJob.addBaggageParameter("trustPUSitelists", False)
        self.dummyJob.addBaggageParameter("skipPileupEvents", 20000)

        baggage = self.dummyJob.getBaggage()
        self.assertTrue(getattr(baggage, "baggageContents"))
        self.assertFalse(getattr(baggage, "trustPUSitelists"))
        self.assertEqual(getattr(baggage, "skipPileupEvents"), 20000)
        self.assertFalse(hasattr(baggage, "IDontExist"))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# coding=utf-8
"""
Wrapper class, based on the PyCuRL module, providing an interface
to CERN MonIT Grafana APIs
"""
from __future__ import division, print_function, absolute_import
from builtins import str, object
from future import standard_library
standard_library.install_aliases()

import logging
import json
from copy import copy
from pprint import pformat
from urllib.parse import urljoin

from WMCore.Services.pycurl_manager import RequestHandler

class Grafana(object):
    """
    Service class providing functionality to CERN monitoring Grafana APIs
    """

    def __init__(self, token, configDict=None):
        """
        Constructs a Grafana object.
        :param token: mandatory string with the key/token string
        :param configDict: dictionary with extra parameters, such as:
          logger: logger object
          endpoint: string with the url/endpoint to be used
          cacheduration: float with the cache duration, in hours
          headers: dictionary with the headers to be used
        """
        self._token = token
        self.configDict = configDict or {}
        self.configDict.setdefault('endpoint', "https://monit-grafana.cern.ch")
        self.configDict.setdefault('cacheduration', 0)  # in hours
        if 'headers' not in self.configDict:
            self.configDict.setdefault('headers', {})
            self.configDict['headers'].update({"Accept": "application/json"})
            self.configDict['headers'].update({"Content-Type": "application/json"})

        self.logger = self.configDict.get("logger", logging.getLogger())
        self.logger.info("MonIT service initialized with parameters: %s", self.configDict)

    def updateToken(self, token):
        """
        Update token to be used in requests made through this module
        :param token: string with the new token
        """
        self._token = token

    def _postRequest(self, url, params, verb='POST', verbose=0):
        "Helper function to POST request to given URL"
        mgr = RequestHandler(logger=self.logger)
        headers = copy(self.configDict['headers'])
        headers.update({"Authorization": self._token})

        try:
            data = mgr.getdata(url, params, headers, verb=verb, verbose=verbose)
            return json.loads(data)
        except Exception as exc:
            self.logger.error("Failed to retrieve data from MonIT. Error: %s", str(exc))
            return None

    def getAPIData(self, apiName, queryStr=''):
        """
        _getAPIData_

        Retrieve data from a given Grafana API and index name
        :param apiName: string with the API name/number
        :param queryStr: string with the whole query logic
        :return: data as retrieved from Grafana
        """
        uri = urljoin(self.configDict['endpoint'], "/api/datasources/proxy/%s/_msearch" % apiName)
        return self._postRequest(uri, queryStr)

    def getSSBData(self, pathName, metricName, indexName="monit_prod_cmssst_*",
                   apiName="9475", queryStr=''):
        """
        _getSSBData_

        Retrieve data from the SSB index in Grafana
        :param pathName: string with the path to the correct metric, mapped from SSB as:
            columnid=237   ProdStatus   path="sts15min"  data.prod_status
            columnid=160   CPU bound    path="scap15min" data.core_cpu_intensive
            columnid=161   I/O bound    path="scap15min" data.core_io_intensive
            columnid=159   Prod Cores   path="scap15min" data.core_production
            columnid=136   Real Cores   path="scap15min" data.core_max_used
            columnid=107   Tape Pledge  path="scap15min" data.tape_pledge
        :param indexName: optional string with the grafana index name
        :param apiName: optional string with the Grafana API number
        :param queryStr: optional string with the full query logic
        :return: dictionary with the site names and the metric value
        """
        # TODO: allow flexible time range instead of the hardwired last 24h
        # TODO: allow flexible number of rows, instead of the hardwired 500
        results = {}
        if metricName not in ('prod_status', 'core_cpu_intensive', 'core_io_intensive',
                              'core_production', 'core_max_used', 'tape_pledge'):
            self.logger.error("SSB metric name '%s' is NOT supported.", metricName)
            return results

        uri = urljoin(self.configDict['endpoint'], "/api/datasources/proxy/%s/_msearch" % apiName)
        if not queryStr:
            ### NOTE: these '\n' new lines are mandatory to get it working...
            queryStr = '{"search_type":"query_then_fetch","ignore_unavailable":true,"index":["%s"]}\n' % indexName
            queryStr += '{"size":500,"query":{"bool":{"filter":[{"range":{"metadata.timestamp":{"gte":"now-1d","lte":"now","format":"epoch_millis"}}},' \
                        '{"query_string":{"analyze_wildcard":true,' \
                        '"query":"metadata.type: ssbmetric AND metadata.type_prefix:raw AND metadata.path: %s"}}]}},' \
                        '"sort":{"metadata.timestamp":{"order":"desc","unmapped_type":"boolean"}},' \
                        '"script_fields":{},"docvalue_fields":["metadata.timestamp"]}\n' % pathName
        self.logger.info("Calling API: %s with query: %s", uri, pformat(queryStr))

        data = self._postRequest(uri, queryStr)
        #self.logger.debug("Data retrieved: %s", pformat(data))
        if data:
            # now parse the ugly output
            hits = data['responses'][0]['hits']['hits']
            for item in hits:
                site = item['_source']['data']['name']
                metricValue = item['_source']['data'][metricName]
                tStamp = int(item['sort'][0])

                results.setdefault(site, {})
                if tStamp > results[site].get("timeStamp", 0):
                    results[site][metricName] = metricValue
                    results[site]["timeStamp"] = tStamp

            data = results

        return data



#!/usr/bin/env python
"""
DBSBufferBlock

This is a block object which will be uploaded to DBS
"""

from builtins import object
import time
import logging
import copy

from WMCore import Lexicon
from WMCore.Services.Requests import JSONRequests
from WMCore.WMException import WMException



class DBSBufferBlockException(WMException):
    """
    _DBSBufferBlockException_

    Container class for exceptions for the DBSBufferBlock
    """



class DBSBufferBlock(object):
    """
    _DBSBufferBlock_

    """

    def __init__(self, name, location, datasetpath):
        """
        Just the necessary objects

        Expects:
          name:  The blockname in full
          location: The PNN of the site the block is at
        """


        self.data      = {'dataset_conf_list':    [],   # List of dataset configurations
                          'file_conf_list':       [],   # List of files, the configuration for each
                          'files':                [],   # List of file objects
                          'block':                {},   # Dict of block info
                          'processing_era':       {},   # Dict of processing era info
                          'acquisition_era':      {},   # Dict of acquisition era information
                          'primds':               {},   # Dict of primary dataset info
                          'dataset':              {},   # Dict of processed dataset info
                          'file_parent_list':     [],   # List of file parents
                          'dataset_parent_list':  [],   # List of parent datasets (DBS requires this as list although it only allows one parent)
                          'close_settings':       {}}   # Dict of info about block close settings

        self.files        = []
        self.encoder      = JSONRequests()
        self.status       = 'Open'
        self.inBuff       = False
        self.startTime    = time.time()
        self.name         = name
        self.location     = location
        self.datasetpath  = datasetpath
        self.workflows    = set()

        self.data['block']['block_name']       = name
        self.data['block']['origin_site_name'] = location
        self.data['block']['open_for_writing'] = 1

        self.data['block']['create_by'] = "WMAgent"
        self.data['block']['creation_date'] = int(time.time())
        self.data['block']['block_size'] = 0
        self.data['block']['file_count'] = 0
        self.data['block']['block_events'] = 0

        self.data['close_settings'] = {}
        self.data['close_settings']['block_close_max_wait_time'] = None
        self.data['close_settings']['block_close_max_events'] = None
        self.data['close_settings']['block_close_max_size'] = None
        self.data['close_settings']['block_close_max_files'] = None
        return


    def encode(self):
        """
        _encode_

        Turn this into a JSON object for transmission
        to DBS
        """

        return self.encoder.encode(data = self.data)



    def addFile(self, dbsFile, datasetType, primaryDatasetType):
        """
        _addFile_

        Add a DBSBufferFile object to our block
        """
        if dbsFile['id'] in [x['id'] for x in self.files]:
            msg =  "Duplicate file inserted into DBSBufferBlock: %i\n" % (dbsFile['id'])
            msg += "Ignoring this file for now!\n"
            logging.error(msg)
            logging.debug("Block length: %i", len(self.files))
            l = sorted([x['id'] for x in self.files])
            logging.debug("First file: %s    Last file: %s", l[0], l[-1])
            return

        for setting in self.data['close_settings']:
            if self.data['close_settings'][setting] is None:
                self.data['close_settings'][setting] = dbsFile[setting]

        self.workflows.add(dbsFile['workflow'])

        self.files.append(dbsFile)
        self.data['block']['block_size'] += int(dbsFile['size'])
        self.data['block']['file_count'] += 1
        self.data['block']['block_events'] += int(dbsFile['events'])

        # Assemble information for the file itself
        fileDict = {}
        fileDict['file_type']              =  'EDM'
        fileDict['logical_file_name']      = dbsFile['lfn']
        fileDict['file_size']              = dbsFile['size']
        fileDict['event_count']            = dbsFile['events']
        fileDict['last_modified_by'] = "WMAgent"
        fileDict['last_modification_date'] = int(time.time())
        fileDict['auto_cross_section'] = 0.0

        # Do the checksums
        for cktype in dbsFile['checksums']:
            cksum = dbsFile['checksums'][cktype]
            if cktype.lower() == 'cksum':
                fileDict['check_sum'] = cksum
            elif cktype.lower() == 'adler32':
                fileDict['adler32'] = cksum
            elif cktype.lower() == 'md5':
                fileDict['md5'] = cksum

        # Do the runs
        lumiList = []
        for run in dbsFile.getRuns():
            for lumi in run.lumis:
                dbsLumiDict = {'lumi_section_num': lumi, 'run_num': run.run}
                if run.getEventsByLumi(lumi) is not None:
                    # if events is not None update event for dbs upload
                    dbsLumiDict['event_count'] = run.getEventsByLumi(lumi)
                lumiList.append(dbsLumiDict)
        fileDict['file_lumi_list'] = lumiList

        # Append to the files list
        self.data['files'].append(fileDict)

        # If dataset_parent_list is defined don't add the file parentage.
        # This means it is block from StepChain workflow and parentage of file will be resloved later
        if not self.data['dataset_parent_list']:
            # now add file to data
            parentLFNs = dbsFile.getParentLFNs()
            for lfn in parentLFNs:
                self.addFileParent(child = dbsFile['lfn'], parent = lfn)


        # Do the algo
        algo = self.addConfiguration(release = dbsFile['appVer'],
                                     psetHash = dbsFile['psetHash'],
                                     appName = dbsFile['appName'],
                                     outputLabel = dbsFile['appFam'],
                                     globalTag = dbsFile['globalTag'])

        # Now add the file with the algo
        # Try to avoid messing with pointers here
        fileAlgo = {}
        fileAlgo.update(algo)
        fileAlgo['lfn'] = dbsFile['lfn']
        self.data['file_conf_list'].append(fileAlgo)

        if dbsFile.get('acquisition_era', False):
            self.setAcquisitionEra(dbsFile['acquisition_era'])
        elif dbsFile.get('acquisitionEra', False):
            self.setAcquisitionEra(dbsFile['acquisitionEra'])
        if dbsFile.get('processingVer', False):
            self.setProcessingVer(dbsFile['processingVer'])
        elif dbsFile.get('processing_ver', False):
            self.setProcessingVer(dbsFile['processing_ver'])

        return

    def addFileParent(self, child, parent):
        """
        _addFileParent_

        Add file parents to the data block
        """
        info = {'parent_logical_file_name': parent,
                'logical_file_name': child}
        self.data['file_parent_list'].append(info)

        return

    def addBlockParent(self, parent):
        """
        _addBlockParent_

        Add the parents of the block
        """

        self.data['block_parent_list'].append({'block_name': parent})
        return

    def addDatasetParent(self, parent):
        """
        _addDatasetParent_

        Add the parent datasets to the data block
        """
        self.data['dataset_parent_list'].append(parent)
        return

    def setProcessingVer(self, procVer):
        """
        _setProcessingVer_

        Set the block's processing version.
        """
        pver = procVer or 0
        try:
            pver = int(pver)
        except TypeError:
            msg = "Provided procVer=%s of type %s cannot be converted to int" \
                    % (procVer, type(procVer))
            raise TypeError(msg) from None
        self.data["processing_era"]["processing_version"] = pver
        self.data["processing_era"]["create_by"] = "WMAgent"
        self.data["processing_era"]["description"] = ""
        return

    def setAcquisitionEra(self, era, date = 123456789):
        """
        _setAcquisitionEra_

        Set the acquisition era for the block
        """
        self.data['acquisition_era']['acquisition_era_name'] = era
        self.data['acquisition_era']['start_date']           = date
        return

    def setPhysicsGroup(self, group):
        """
        _setPhysicsGroup_

        Sets the name of the physics group to which the dataset is attached
        """

        self.data['dataset']['physics_group_name'] = group
        return

    def getDatasetPath(self):
        """
        _getDatasetPath_

        Return the datasetpath
        """
        return self.datasetpath

    def getDataset(self):
        """
        _getDataset_

        Return the dataset (None if not set)
        """
        return self.data['dataset'].get('dataset', None)

    def setDataset(self, datasetName, primaryType,
                   datasetType, physicsGroup = None,
                   prep_id  = None, overwrite = False):
        """
        _setDataset_

        Set all the information concerning a single dataset, including
        the primary, processed and tier info
        """
        if self.getDataset() != None and not overwrite:
            # Do nothing, we already have a dataset
            return

        Lexicon.primaryDatasetType(primaryType)

        _, primary, processed, tier = datasetName.split('/')

        # Do the primary dataset
        self.data['primds']['primary_ds_name'] = primary
        self.data['primds']['primary_ds_type'] = primaryType
        self.data['primds']['create_by'] = "WMAgent"
        self.data['primds']['creation_date'] = int(time.time())

        # Do the processed
        self.data['dataset']['physics_group_name']  = physicsGroup
        self.data['dataset']['processed_ds_name']   = processed
        self.data['dataset']['data_tier_name']      = tier
        self.data['dataset']['dataset_access_type'] = datasetType
        self.data['dataset']['dataset']             = datasetName
        self.data['dataset']['prep_id'] = prep_id
        # Add misc meta data.
        self.data['dataset']['create_by'] = "WMAgent"
        self.data['dataset']['last_modified_by'] = "WMAgent"
        self.data['dataset']['creation_date'] = int(time.time())
        self.data['dataset']['last_modification_date'] = int(time.time())
        return


    def addConfiguration(self, release, psetHash,
                         appName = 'cmsRun', outputLabel = 'Merged', globalTag = 'None'):
        """
        _addConfiguration_

        Add the algorithm config to the data block
        """

        algo = {'release_version': release,
                'pset_hash': psetHash,
                'app_name': appName,
                'output_module_label': outputLabel,
                'global_tag': globalTag}

        if not algo in self.data['dataset_conf_list']:
            self.data['dataset_conf_list'].append(algo)


        return algo

    def getNFiles(self):
        """
        _getNFiles_

        Return the number of files in the block
        """

        return len(self.files)


    def getSize(self):
        """
        _getSize_

        Get size of block
        """
        return self.data['block']['block_size']

    def getNumEvents(self):
        """
        _getNumEvents_

        Get the number of events in the block
        """
        return self.data['block']['block_events']

    def getTime(self):
        """
        _getTime_

        Return the time the block has been running
        """

        return time.time() - self.startTime

    def getMaxBlockTime(self):
        """
        _getMaxBlockTime_

        Return the max time that the block should stay open
        """
        return self.data['close_settings']['block_close_max_wait_time']

    def getMaxBlockSize(self):
        """
        _getMaxBlockSize_

        Return the max size allowed for the block
        """
        return self.data['close_settings']['block_close_max_size']

    def getMaxBlockNumEvents(self):
        """
        _getMaxBlockNumEvents_

        Return the max number of events allowed for the block
        """
        return self.data['close_settings']['block_close_max_events']

    def getMaxBlockFiles(self):
        """
        _getMaxBlockFiles_

        Return the max number of files allowed for the block
        """
        return self.data['close_settings']['block_close_max_files']

    def getName(self):
        """
        _getName_

        Get Name
        """

        return self.name

    def getLocation(self):
        """
        _getLocation_

        Get location
        """

        return self.location

    def getStartTime(self):
        """
        _getStartTime_

        Get the time the block was opened at
        """

        return self.startTime


    def FillFromDBSBuffer(self, blockInfo):
        """
        _FillFromDBSBuffer_

        Take the info provided by LoadBlocks and
        use it to create a block object
        """
        # Blocks loaded out of the buffer should
        # have both a creation time, and should
        # be in the buffer (duh)
        self.startTime = blockInfo.get('creation_date')
        self.inBuff    = True

        if 'status' in blockInfo:
            self.status = blockInfo['status']
            if self.status == "Pending":
                self.data['block']['open_for_writing'] = 0

            del blockInfo['status']

        for key in blockInfo:
            self.data['block'][key] = blockInfo.get(key)

    def convertToDBSBlock(self):
        """
        convert to DBSBlock structure to upload to dbs
        TODO: check file lumi event and validate event is not null
        """
        block = {}

        #TODO: instead of using key to remove need to change to keyToKeep
        # Ask dbs team to publish the list (API)
        keyToRemove = ['insertedFiles', 'newFiles', 'file_count', 'block_size',
                       'origin_site_name', 'creation_date', 'open',
                       'Name', 'close_settings']

        nestedKeyToRemove = ['block.block_events', 'block.datasetpath', 'block.workflows']

        dbsBufferToDBSBlockKey = {'block_size': 'BlockSize',
                                  'creation_date': 'CreationDate',
                                  'file_count': 'NumberOfFiles',
                                  'origin_site_name': 'location'}

        # clone the new DBSBlock dict after filtering out the data.
        for key in self.data:
            if key in keyToRemove:
                continue
            elif key in dbsBufferToDBSBlockKey:
                block[dbsBufferToDBSBlockKey[key]] = copy.deepcopy(self.data[key])
            else:
                block[key] = copy.deepcopy(self.data[key])

        # delete nested key dictionary
        for nestedKey in nestedKeyToRemove:
            firstkey, subkey = nestedKey.split('.', 1)
            if firstkey in block and subkey in block[firstkey]:
                del block[firstkey][subkey]

        return block

    def setPendingAndCloseBlock(self):
        "set the block status as Pending for upload as well as closed"
        # Pending means ready to upload
        self.status = "Pending"
        # close block on DBS3 status
        self.data['block']['open_for_writing'] = 0



#!/usr/bin/env python
"""
_Configuration_

Module dealing with Agent Configuration file in python format

This code has been moded to WMCore.Configuration


"""

import os
import imp
import types

from WMCore.Configuration import ConfigSection as BaseConfigSection
from WMCore.Configuration import Configuration as BaseConfiguration
from WMCore.Configuration import loadConfigurationFile as baseLoadConfigurationFile
from WMCore.Configuration import saveConfigurationFile as baseSaveConfigurationFile


class ConfigSection(BaseConfigSection):
    """
    _ConfigSection_

    Chunk of configuration information

    """
    def __init__(self, name = None):
        BaseConfigSection.__init__(self, name)


class Configuration(BaseConfiguration):
    """
    _Configuration_

    Top level configuration object

    """
    def __init__(self):
        BaseConfiguration.__init__(self)
        self._internal_sections.append("Agent")
        self.Agent = ConfigSection("Agent")
        self.Agent.agentName = None
        self.Agent.teamName = None
        self.Agent.hostName = None
        self.Agent.contact = None

def loadConfigurationFile(filename):
    """
    _loadConfigurationFile_

    Load a Configuration File

    """
    assert filename != ""
    return baseLoadConfigurationFile(filename)

def saveConfigurationFile(configInstance, filename, **options):
    """
    _saveConfigurationFile_

    Save the configuration as a python module
    Options controls the format of documentation

    comment = True means save docs as comments
    document = True means save docs as document_ calls


    """
    baseSaveConfigurationFile(configInstance, filename, **options)



#!/usr/bin/env python
from __future__ import print_function, division
import unittest
import time

from Utils.PythonVersion import PY3

from WMCore.WorkQueue.WorkQueue import globalQueue
from WMCore.WorkQueue.WorkQueue import localQueue
from WMCore.Services.WorkQueue.WorkQueue import WorkQueue as WorkQueueDS
from WMCore.Services.WorkQueue.WorkQueue import convertWQElementsStatusToWFStatus
from WMQuality.Emulators.WMSpecGenerator.WMSpecGenerator import WMSpecGenerator
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMQuality.TestInitCouchApp import TestInitCouchApp

class WorkQueueTest(EmulatedUnitTestCase):
    """
    Test WorkQueue Service client
    It will start WorkQueue RESTService
    Server DB sets from environment variable.
    Client DB sets from environment variable.

    This checks whether DS call makes without error and return the results.
    Not the correctness of functions. That will be tested in different module.
    """

    def setUp(self):
        """
        _setUp_
        """
        super(WorkQueueTest, self).setUp()

        self.specGenerator = WMSpecGenerator("WMSpecs")
        # self.configFile = EmulatorSetup.setupWMAgentConfig()
        self.schema = []
        self.couchApps = ["WorkQueue"]
        self.testInit = TestInitCouchApp('WorkQueueServiceTest')
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=self.schema,
                                useDefault=False)
        self.testInit.setupCouch('workqueue_t', *self.couchApps)
        self.testInit.setupCouch('workqueue_t_inbox', *self.couchApps)
        self.testInit.setupCouch('local_workqueue_t', *self.couchApps)
        self.testInit.setupCouch('local_workqueue_t_inbox', *self.couchApps)
        self.testInit.generateWorkDir()

        # setup rucio parameters for global/local queue
        self.queueParams = {}
        self.queueParams['log_reporter'] = "Services_WorkQueue_Unittest"
        self.queueParams['rucioAccount'] = "wma_test"
        self.queueParams['rucioAuthUrl'] = "http://cms-rucio-int.cern.ch"
        self.queueParams['rucioUrl'] = "https://cms-rucio-auth-int.cern.ch"

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.tearDownCouch()
        self.specGenerator.removeSpecs()
        super(WorkQueueTest, self).tearDown()

    def testWorkQueueService(self):
        # test getWork
        specName = "RerecoSpec"
        specUrl = self.specGenerator.createReRecoSpec(specName, "file",
                                                      assignKwargs={'SiteWhitelist': ['T2_XX_SiteA']})
        globalQ = globalQueue(DbName='workqueue_t',
                              QueueURL=self.testInit.couchUrl,
                              UnittestFlag=True,
                              **self.queueParams)
        self.assertTrue(globalQ.queueWork(specUrl, specName, "teamA") > 0)

        wqApi = WorkQueueDS(self.testInit.couchUrl, 'workqueue_t')
        # overwrite default - can't test with stale view
        wqApi.defaultOptions = {'reduce': True, 'group': True}
        # This only checks minimum client call not exactly correctness of return
        # values.
        self.assertEqual(wqApi.getTopLevelJobsByRequest(),
                         [{'total_jobs': 339, 'request_name': specName}])
        # work still available, so no childQueue
        results = wqApi.getChildQueuesAndStatus()
        self.assertItemsEqual(set([item['agent_name'] for item in results]), ["AgentNotDefined"])
        result = wqApi.getElementsCountAndJobsByWorkflow()
        self.assertEqual(len(result), 1)
        self.assertEqual(result[specName]['Available']['Jobs'], 339)

        results = wqApi.getChildQueuesAndPriority()
        resultsPrio = set([item['priority'] for item in results if item['agent_name'] == "AgentNotDefined"])
        self.assertItemsEqual(resultsPrio, [8000])
        self.assertEqual(wqApi.getWMBSUrl(), [])
        self.assertEqual(wqApi.getWMBSUrlByRequest(), [])

    def testUpdatePriorityService(self):
        """
        _testUpdatePriorityService_

        Check that we can update the priority correctly also
        check the available workflows feature
        """
        specName = "RerecoSpec"
        specUrl = self.specGenerator.createReRecoSpec(specName, "file",
                                                      assignKwargs={'SiteWhitelist':["T2_XX_SiteA"]})
        globalQ = globalQueue(DbName='workqueue_t',
                              QueueURL=self.testInit.couchUrl,
                              UnittestFlag=True,
                              **self.queueParams)
        localQ = localQueue(DbName='local_workqueue_t',
                            QueueURL=self.testInit.couchUrl,
                            CacheDir=self.testInit.testDir,
                            ParentQueueCouchUrl='%s/workqueue_t' % self.testInit.couchUrl,
                            ParentQueueInboxCouchDBName='workqueue_t_inbox',
                            **self.queueParams)
        # Try a full chain of priority update and propagation
        self.assertTrue(globalQ.queueWork(specUrl, "RerecoSpec", "teamA") > 0)
        globalApi = WorkQueueDS(self.testInit.couchUrl, 'workqueue_t')
        # overwrite default - can't test with stale view
        globalApi.defaultOptions = {'reduce': True, 'group': True}
        globalApi.updatePriority(specName, 100)
        self.assertEqual(globalQ.backend.getWMSpec(specName).priority(), 100)
        storedElements = globalQ.backend.getElementsForWorkflow(specName)
        for element in storedElements:
            self.assertEqual(element['Priority'], 100)
        numWorks = localQ.pullWork({'T2_XX_SiteA': 10})
        self.assertTrue(numWorks > 0)
        # replicate from GQ to LQ manually
        localQ.backend.pullFromParent(continuous=False)
        # wait until replication is done
        time.sleep(2)

        localQ.processInboundWork(continuous=False)
        storedElements = localQ.backend.getElementsForWorkflow(specName)
        for element in storedElements:
            self.assertEqual(element['Priority'], 100)
        localApi = WorkQueueDS(self.testInit.couchUrl, 'local_workqueue_t')
        # overwrite default - can't test with stale view
        localApi.defaultOptions = {'reduce': True, 'group': True}
        localApi.updatePriority(specName, 500)
        self.assertEqual(localQ.backend.getWMSpec(specName).priority(), 500)
        storedElements = localQ.backend.getElementsForWorkflow(specName)
        for element in storedElements:
            self.assertEqual(element['Priority'], 500)
        availableWF = localApi.getAvailableWorkflows()
        self.assertEqual(availableWF, set([(specName, 500)]))
        # Attempt to update an inexistent workflow in the queue
        try:
            globalApi.updatePriority('NotExistent', 2)
        except Exception as ex:
            self.fail('No exception should be raised.: %s' % str(ex))

    def testCompletedWorkflow(self):
        # test getWork
        specName = "RerecoSpec"
        specUrl = self.specGenerator.createReRecoSpec(specName, "file",
                                                      assignKwargs={'SiteWhitelist':['T2_XX_SiteA']})

        globalQ = globalQueue(DbName='workqueue_t',
                              QueueURL=self.testInit.couchUrl,
                              UnittestFlag=True,
                              **self.queueParams)
        self.assertTrue(globalQ.queueWork(specUrl, specName, "teamA") > 0)

        wqApi = WorkQueueDS(self.testInit.couchUrl, 'workqueue_t')
        # overwrite default - can't test with stale view
        wqApi.defaultOptions = {'reduce': True, 'group': True}
        # This only checks minimum client call not exactly correctness of return
        # values.
        self.assertEqual(wqApi.getTopLevelJobsByRequest(),
                         [{'total_jobs': 339, 'request_name': specName}])

        results = wqApi.getJobsByStatus()
        self.assertEqual(results['Available']['sum_jobs'], 339)
        results = wqApi.getJobsByStatusAndPriority()
        resultsPrio = set([item['priority'] for item in results.get('Available')])
        self.assertItemsEqual(resultsPrio, [8000])
        resultsJobs = sum([item['sum_jobs'] for item in results.get('Available') if item['priority'] == 8000])
        self.assertTrue(resultsJobs, 339)
        result = wqApi.getElementsCountAndJobsByWorkflow()
        self.assertEqual(len(result), 1)
        self.assertEqual(result[specName]['Available']['Jobs'], 339)
        data = wqApi.db.loadView('WorkQueue', 'elementsDetailByWorkflowAndStatus',
                                 {'startkey': [specName], 'endkey': [specName, {}],
                                  'reduce': False})
        elements = [x['id'] for x in data.get('rows', [])]
        wqApi.updateElements(*elements, Status='Canceled')
        # load this view once again to make sure it will be updated in the next assert..
        data = wqApi.db.loadView('WorkQueue', 'elementsDetailByWorkflowAndStatus',
                                 {'startkey': [specName], 'endkey': [specName, {}],
                                  'reduce': False})
        self.assertEqual(len(wqApi.getCompletedWorkflow(stale=False)), 1)
        results = wqApi.getJobsByStatusAndPriority()
        resultsPrio = set([item['priority'] for item in results.get('Canceled')])
        self.assertItemsEqual(resultsPrio, [8000])

    def testConvertWQElementsStatusToWFStatus(self):
        """
        _testConvertWQElementsStatusToWFStatus_

        Check that a set of all the workqueue element status in a request
        properly maps to a single state to trigger the ReqMgr request transition.
        """
        # workflows acquired by global_workqueue (nothing acquired by agents so far)
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating"])), "acquired")

        # workflows not completely acquired yet by the agents
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Acquired"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Acquired", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Acquired", "Running", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired", "Running", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Running", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Running", "Done", "Canceled"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Running", "Done", "Canceled"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Acquired", "Running", "Done", "Canceled"])), "running-open")

        # workflows completely acquired by the agents
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running"])), "running-closed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running", "Done"])), "running-closed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running", "Done", "Canceled"])), "running-closed")

        # workflows completed/aborted/force-completed, thus existent elements
        # but no more active workqueue elements in the system
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Done"])), "completed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Canceled"])), "completed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Done", "Canceled"])), "completed")

        # workflows failed
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Failed"])), "failed")

        # non-failed workflows but with Failed elements
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Acquired", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Acquired", "Running", "Done", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired", "Running", "Done", "Canceled", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running", "Failed"])), "running-closed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running", "Done", "Canceled", "Failed"])), "running-closed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Done", "Failed"])), "completed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Canceled", "Failed"])), "completed")

        # workflows that have been aborted but still have workqueue elements around
        self.assertEqual("running-open", convertWQElementsStatusToWFStatus(
                set(["Available", "Negotiating", "Acquired", "Running", "Done", "CancelRequested"])))
        self.assertEqual("running-open", convertWQElementsStatusToWFStatus(
                set(["Available", "Negotiating", "Acquired", "Running", "Done", "CancelRequested", "Canceled"])))
        self.assertEqual("running-open", convertWQElementsStatusToWFStatus(
                set(["Negotiating", "Acquired", "Running", "Done", "CancelRequested"])))
        self.assertEqual("running-open", convertWQElementsStatusToWFStatus(
                set(["Negotiating", "Acquired", "Running", "Done", "CancelRequested", "Canceled"])))
        self.assertEqual("running-open", convertWQElementsStatusToWFStatus(
                set(["Acquired", "Running", "Done", "CancelRequested"])))
        self.assertEqual("running-open", convertWQElementsStatusToWFStatus(
                set(["Acquired", "Running", "Done", "CancelRequested", "Canceled"])))
        self.assertEqual("running-closed", convertWQElementsStatusToWFStatus(
                set(["Running", "Done", "CancelRequested"])))
        self.assertEqual("running-closed", convertWQElementsStatusToWFStatus(
                set(["Running", "Done", "CancelRequested", "Canceled"])))
        self.assertEqual("canceled", convertWQElementsStatusToWFStatus(
                set(["Done", "CancelRequested"])))
        self.assertEqual("canceled", convertWQElementsStatusToWFStatus(set(["CancelRequested"])))

    def test2ConvertWQElementsStatusToWFStatus(self):
        """
        Same as the test 'testConvertWQElementsStatusToWFStatus', but testing
        'convertWQElementsStatusToWFStatus' function from a different angle.
        """
        # single WQE with standard state transition
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running"])), "running-closed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Done"])), "completed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["CancelRequested"])), "canceled")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Canceled"])), "completed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Failed"])), "failed")

        # double WQE with standard state transition
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Acquired"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Done"])), 'running-open')
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Failed"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Done"])), 'running-open')
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Failed"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Done"])), 'running-open')
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running", "Done"])), 'running-closed')
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running", "Failed"])), "running-closed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Done", "Failed"])), "completed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Done", "CancelRequested"])), "canceled")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Done", "Canceled"])), "completed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["CancelRequested", "Canceled"])), "canceled")

        # triple WQE with standard state transition
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Acquired"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Negotiating", "Failed"])), "acquired")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Acquired", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Acquired", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Acquired", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Running", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Running", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Available", "Done", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired", "Running"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Acquired", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Running", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Running", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Negotiating", "Done", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Running", "Done"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Running", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Acquired", "Done", "Failed"])), "running-open")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Running", "Done", "Failed"])), "running-closed")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["CancelRequested", "Done", "Failed"])), "canceled")
        self.assertEqual(convertWQElementsStatusToWFStatus(set(["Canceled", "Done", "Failed"])), "completed")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_File_t_

Unit tests for the WMBS File class.
"""

import logging
import threading
import unittest

from Utils.PythonVersion import PY3

from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.File import File as WMFile
from WMCore.DataStructs.Run import Run
from WMCore.WMBS.File import File, addFilesToWMBSInBulk
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMQuality.TestInit import TestInit


class FileTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.  Try to create all of the
        WMBS tables.  Also add some dummy locations.
        """

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)

        myThread = threading.currentThread()
        self.daofactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)

        locationAction = self.daofactory(classname="Locations.New")
        locationAction.execute(siteName="site1", pnn="T1_US_FNAL_Disk")
        locationAction.execute(siteName="site2", pnn="T2_CH_CERN")

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

        return

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.clearDatabase()
        return

    def testCreateDeleteExists(self):
        """
        _testCreateDeleteExists_

        Test the create(), delete() and exists() methods of the file class
        by creating and deleting a file.  The exists() method will be
        called before and after creation and after deletion.
        """
        testFile = File(lfn="/this/is/a/lfn", size=1024, events=10, checksums={'cksum': 1111})

        assert testFile.exists() == False, \
            "ERROR: File exists before it was created"

        testFile.addRun(Run(1, *[45]))
        testFile.create()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after it was created"

        testFile.delete()

        assert testFile.exists() == False, \
            "ERROR: File exists after it has been deleted"
        return

    def testCreateTransaction(self):
        """
        _testCreateTransaction_

        Begin a transaction and then create a file in the database.  Afterwards,
        rollback the transaction.  Use the File class's exists() method to
        to verify that the file doesn't exist before it was created, exists
        after it was created and doesn't exist after the transaction was rolled
        back.
        """
        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFile = File(lfn="/this/is/a/lfn", size=1024, events=10, checksums={'cksum': 1111})

        assert testFile.exists() == False, \
            "ERROR: File exists before it was created"

        testFile.addRun(Run(1, *[45]))
        testFile.create()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after it was created"

        myThread.transaction.rollback()

        assert testFile.exists() == False, \
            "ERROR: File exists after transaction was rolled back."
        return

    def testDeleteTransaction(self):
        """
        _testDeleteTransaction_

        Create a file and commit it to the database.  Start a new transaction
        and delete the file.  Rollback the transaction after the file has been
        deleted.  Use the file class's exists() method to verify that the file
        does not exist after it has been deleted but does exist after the
        transaction is rolled back.
        """
        testFile = File(lfn="/this/is/a/lfn", size=1024, events=10,
                        checksums={'cksum': 1111})

        assert testFile.exists() == False, \
            "ERROR: File exists before it was created"

        testFile.addRun(Run(1, *[45]))
        testFile.create()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after it was created"

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFile.delete()

        assert testFile.exists() == False, \
            "ERROR: File exists after it has been deleted"

        myThread.transaction.rollback()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after transaction was rolled back."
        return

    def testGetInfo(self):
        """
        _testGetInfo_

        Test the getInfo() method of the File class to make sure that it
        returns the correct information.
        """
        testFileParent = File(lfn="/this/is/a/parent/lfn", size=1024,
                              events=20, checksums={'cksum': 1111})
        testFileParent.addRun(Run(1, *[45]))
        testFileParent.create()

        testFile = File(lfn="/this/is/a/lfn", size=1024, events=10, checksums={'cksum': 222})
        testFile.addRun(Run(1, *[45]))
        testFile.addRun(Run(2, *[46, 47]))
        testFile.addRun(Run(2, *[47, 48]))
        testFile.create()
        testFile.setLocation(pnn="T1_US_FNAL_Disk", immediateSave=False)
        testFile.setLocation(pnn="T2_CH_CERN", immediateSave=False)
        testFile.addParent("/this/is/a/parent/lfn")

        info = testFile.getInfo()

        assert info[0] == testFile["lfn"], \
            "ERROR: File returned wrong LFN"

        assert info[1] == testFile["id"], \
            "ERROR: File returned wrong ID"

        assert info[2] == testFile["size"], \
            "ERROR: File returned wrong size"

        assert info[3] == testFile["events"], \
            "ERROR: File returned wrong events"

        assert info[4] == testFile["checksums"], \
            "ERROR: File returned wrong cksum"

        assert len(info[5]) == 2, \
            "ERROR: File returned wrong runs"

        self.assertItemsEqual(info[5], [Run(1, *[45]), Run(2, *[46, 47, 48])],
                              "Error: Run hasn't been combined correctly")

        assert len(info[6]) == 2, \
            "ERROR: File returned wrong locations"

        for testLocation in info[6]:
            assert testLocation in ["T1_US_FNAL_Disk", "T2_CH_CERN"], \
                "ERROR: File returned wrong locations"

        assert len(info[7]) == 1, \
            "ERROR: File returned wrong parents"

        assert info[7][0] == testFileParent, \
            "ERROR: File returned wrong parents"

        testFile.delete()
        testFileParent.delete()
        return

    def testGetParentLFNs(self):
        """
        _testGetParentLFNs_

        Create three files and set them to be parents of a fourth file.  Check
        to make sure that getParentLFNs() on the child file returns the correct
        LFNs.
        """
        testFileParentA = File(lfn="/this/is/a/parent/lfnA", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentA.addRun(Run(1, *[45]))
        testFileParentB = File(lfn="/this/is/a/parent/lfnB", size=1024,
                               events=20, checksums={'cksum': 2})
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentC = File(lfn="/this/is/a/parent/lfnC", size=1024,
                               events=20, checksums={'cksum': 3})
        testFileParentC.addRun(Run(1, *[45]))

        testFileParentA.create()
        testFileParentB.create()
        testFileParentC.create()

        testFile = File(lfn="/this/is/a/lfn", size=1024,
                        events=10, checksums={'cksum': 1})
        testFile.addRun(Run(1, *[45]))
        testFile.create()

        testFile.addParent(testFileParentA["lfn"])
        testFile.addParent(testFileParentB["lfn"])
        testFile.addParent(testFileParentC["lfn"])

        parentLFNs = testFile.getParentLFNs()

        assert len(parentLFNs) == 3, \
            "ERROR: Child does not have the right amount of parents"

        goldenLFNs = ["/this/is/a/parent/lfnA",
                      "/this/is/a/parent/lfnB",
                      "/this/is/a/parent/lfnC"]
        for parentLFN in parentLFNs:
            assert parentLFN in goldenLFNs, \
                "ERROR: Unknown parent lfn"
            goldenLFNs.remove(parentLFN)

        testFile.delete()
        testFileParentA.delete()
        testFileParentB.delete()
        testFileParentC.delete()
        return

    def testLoad(self):
        """
        _testLoad_

        Test the loading of file meta data using the ID of a file and the
        LFN of a file.
        """
        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 101}, first_event=2, merged=True)
        testFileA.create()

        testFileB = File(lfn=testFileA["lfn"])
        testFileB.load()
        testFileC = File(id=testFileA["id"])
        testFileC.load()

        self.assertEqual(testFileA, testFileB, "File load by LFN didn't work")
        self.assertEqual(testFileA, testFileC, "File load by ID didn't work")

        self.assertTrue(isinstance(testFileB["id"], int), "File id is not an integer type.")
        self.assertTrue(isinstance(testFileB["size"], int), "File size is not an integer type.")
        self.assertTrue(isinstance(testFileB["events"], int), "File events is not an integer type.")
        self.assertTrue(isinstance(testFileB["checksums"], dict), "File cksum is not a string type.")
        self.assertTrue(isinstance(testFileB["first_event"], int), "File first_event is not an integer type.")

        self.assertTrue(isinstance(testFileC["id"], int), "File id is not an integer type.")
        self.assertTrue(isinstance(testFileC["size"], int), "File size is not an integer type.")
        self.assertTrue(isinstance(testFileC["events"], int), "File events is not an integer type.")
        self.assertTrue(isinstance(testFileC["checksums"], dict), "File cksum is not an string type.")
        self.assertTrue(isinstance(testFileC["first_event"], int), "File first_event is not an integer type.")

        self.assertEqual(testFileC['checksums'], {'cksum': '101'})

        testFileA.delete()
        return

    def testLoadData(self):
        """
        _testLoadData_

        Test the loading of all data from a file, including run/lumi
        associations, location information and parentage information.
        """
        testFileParentA = File(lfn="/this/is/a/parent/lfnA", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentA.addRun(Run(1, *[45]))
        testFileParentB = File(lfn="/this/is/a/parent/lfnB", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentA.create()
        testFileParentB.create()

        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()
        testFileA.setLocation(pnn="T1_US_FNAL_Disk", immediateSave=False)
        testFileA.setLocation(pnn="T2_CH_CERN", immediateSave=False)
        testFileA.addParent("/this/is/a/parent/lfnA")
        testFileA.addParent("/this/is/a/parent/lfnB")
        testFileA.updateLocations()

        testFileB = File(lfn=testFileA["lfn"])
        testFileB.loadData(parentage=1)
        testFileC = File(id=testFileA["id"])
        testFileC.loadData(parentage=1)

        assert testFileA == testFileB, \
            "ERROR: File load by LFN didn't work"

        assert testFileA == testFileC, \
            "ERROR: File load by ID didn't work"

        testFileA.delete()
        testFileParentA.delete()
        testFileParentB.delete()
        return

    def testAddChild(self):
        """
        _testAddChild_

        Add a child to some parent files and make sure that all the parentage
        information is loaded/stored correctly from the database.
        """
        testFileParentA = File(lfn="/this/is/a/parent/lfnA", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentA.addRun(Run(1, *[45]))
        testFileParentB = File(lfn="/this/is/a/parent/lfnB", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentA.create()
        testFileParentB.create()

        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileParentA.addChild("/this/is/a/lfn")
        testFileParentB.addChild("/this/is/a/lfn")

        testFileB = File(id=testFileA["id"])
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"
        return

    def testAddChildTransaction(self):
        """
        _testAddChildTransaction_

        Add a child to some parent files and make sure that all the parentage
        information is loaded/stored correctly from the database.  Rollback the
        addition of one of the childs and then verify that it does in fact only
        have one parent.
        """
        testFileParentA = File(lfn="/this/is/a/parent/lfnA", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentA.addRun(Run(1, *[45]))
        testFileParentB = File(lfn="/this/is/a/parent/lfnB", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentA.create()
        testFileParentB.create()

        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileParentA.addChild("/this/is/a/lfn")

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFileParentB.addChild("/this/is/a/lfn")

        testFileB = File(id=testFileA["id"])
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"

        myThread.transaction.rollback()
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA]
        for parentFile in testFileB["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"

        return

    def testCreateWithLocation(self):
        """
        _testCreateWithLocation_

        Create a file and add a couple locations.  Load the file from the
        database to make sure that the locations were set correctly.
        """
        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1},
                         locations=set(["T1_US_FNAL_Disk", "T2_CH_CERN"]))
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileB = File(id=testFileA["id"])
        testFileB.loadData()

        goldenLocations = ["T1_US_FNAL_Disk", "T2_CH_CERN"]

        for location in testFileB["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"
        return

    def testSetLocation(self):
        """
        _testSetLocation_

        Create a file and add a couple locations.  Load the file from the
        database to make sure that the locations were set correctly.
        """
        testFile = File(lfn="/this/is/a/lfn1", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T2_CH_CERN")
        self.assertItemsEqual(testFile["newlocations"], set(["T2_CH_CERN"]))
        testFile.setLocation(u"T1_US_FNAL_Disk")
        self.assertItemsEqual(testFile["newlocations"], set(["T2_CH_CERN", "T1_US_FNAL_Disk"]))

        testFile = File(lfn="/this/is/a/lfn2", size=1024, events=10,
                         checksums={'cksum': 1}, locations=u"T2_CH_CERN")
        self.assertItemsEqual(testFile["newlocations"], set(["T2_CH_CERN"]))
        testFile.setLocation(b"T1_US_FNAL_Disk")
        self.assertItemsEqual(testFile["newlocations"], set(["T2_CH_CERN", b"T1_US_FNAL_Disk"]))

        testFile = File(lfn="/this/is/a/lfn3", size=1024, events=10,
                         checksums={'cksum': 1}, locations=set(["T2_CH_CERN"]))
        self.assertItemsEqual(testFile["newlocations"], set(["T2_CH_CERN"]))
        testFile.setLocation("T1_US_FNAL_Disk")
        self.assertItemsEqual(testFile["newlocations"], set(["T2_CH_CERN", "T1_US_FNAL_Disk"]))

        testFileA = File(lfn="/this/is/a/lfn4", size=1024, events=10,
                         checksums={'cksum': 1})
        self.assertItemsEqual(testFileA["newlocations"], set([]))
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()
        testFileA.setLocation(["T1_US_FNAL_Disk", "T2_CH_CERN"], immediateSave=False)
        self.assertItemsEqual(testFileA["newlocations"], set(["T2_CH_CERN", "T1_US_FNAL_Disk"]))
        testFileA.setLocation(["T1_US_FNAL_Disk", "T2_CH_CERN"], immediateSave=True)
        # newlocations is cleared when they are persisted in the database
        self.assertItemsEqual(testFileA["newlocations"], set([]))
        testFileA.setLocation(["bunkT1_US_FNAL_Disk", "bunkT2_CH_CERN"],
                              immediateSave=False)

        testFileB = File(id=testFileA["id"])
        testFileB.loadData()
        self.assertItemsEqual(testFileB["locations"], set(["T2_CH_CERN", "T1_US_FNAL_Disk"]))

    def testSetLocationTransaction(self):
        """
        _testSetLocationTransaction_

        Create a file at specific locations and commit everything to the
        database.  Reload the file from the database and verify that the
        locations are correct.  Rollback the database transaction and once
        again reload the file.  Verify that the original locations are back.
        """
        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()
        testFileA.setLocation(["T1_US_FNAL_Disk"])

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFileA.setLocation(["T2_CH_CERN"])
        testFileA.setLocation(["bunkT1_US_FNAL_Disk", "bunkT2_CH_CERN"],
                              immediateSave=False)

        testFileB = File(id=testFileA["id"])
        testFileB.loadData()

        goldenLocations = ["T1_US_FNAL_Disk", "T2_CH_CERN"]

        for location in testFileB["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"

        myThread.transaction.rollback()
        testFileB.loadData()

        goldenLocations = ["T1_US_FNAL_Disk"]

        for location in testFileB["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"
        return

    def testLocationsConstructor(self):
        """
        _testLocationsConstructor_

        Test to make sure that locations passed into the File() constructor
        are loaded from and save to the database correctly.  Also test to make
        sure that the class behaves well when the location is passed in as a
        single string instead of a set.
        """
        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1}, locations=set(["T1_US_FNAL_Disk"]))
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileB = File(lfn="/this/is/a/lfn2", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFileB.addRun(Run(1, *[45]))
        testFileB.create()

        testFileC = File(id=testFileA["id"])
        testFileC.loadData()

        goldenLocations = ["T1_US_FNAL_Disk"]
        for location in testFileC["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"

        testFileC = File(id=testFileB["id"])
        testFileC.loadData()

        goldenLocations = ["T1_US_FNAL_Disk"]
        for location in testFileC["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"
        return

    def testSetLocationOrder(self):
        """
        _testSetLocationOrder_

        This tests that you can specify a location before creating the file,
        instead of having to do it afterwards.
        """
        myThread = threading.currentThread()

        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setLocation("T2_CH_CERN")
        testFileA.create()

        testFileB = File(lfn=testFileA["lfn"])
        testFileB.load()

        daoFactory = DAOFactory(package="WMCore.WMBS", logger=logging, dbinterface=myThread.dbi)

        locationFac = daoFactory(classname="Files.GetLocation")
        location = locationFac.execute(testFileB['lfn']).pop()

        self.assertEqual(location, 'T2_CH_CERN')

        return

    def testAddRunSet(self):
        """
        _testAddRunSet_

        Test the ability to add run and lumi information to a file.
        """
        testFile = File(lfn="/this/is/a/lfn", size=1024, events=10,
                        checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFile.create()
        runSet = set()
        runSet.add(Run(1, *[45]))
        runSet.add(Run(2, *[67, 68]))
        testFile.addRunSet(runSet)

        assert (runSet - testFile["runs"]) == set(), \
            "Error: addRunSet is not updating set correctly"

        return

    def testGetAncestorLFNs(self):
        """
        _testGenAncestorLFNs_

        Create a series of files that have several generations of parentage
        information.  Verify that the parentage information is reported
        correctly.
        """
        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFileA.create()

        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFileB.create()

        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFileC.create()

        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFileD.create()

        testFileE = File(lfn="/this/is/a/lfnE", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFileE.create()

        testFileE = File(lfn="/this/is/a/lfnF", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T1_US_FNAL_Disk")
        testFileE.create()

        testFileA.addParent(lfn="/this/is/a/lfnB")
        testFileA.addParent(lfn="/this/is/a/lfnC")
        testFileB.addParent(lfn="/this/is/a/lfnD")
        testFileC.addParent(lfn="/this/is/a/lfnD")
        testFileD.addParent(lfn="/this/is/a/lfnE")
        testFileD.addParent(lfn="/this/is/a/lfnF")

        level1 = ["/this/is/a/lfnB", "/this/is/a/lfnC"]
        level2 = ["/this/is/a/lfnD"]
        level3 = ["/this/is/a/lfnE", "/this/is/a/lfnF"]
        level4 = level5 = []

        decs2 = ["/this/is/a/lfnA"]

        assert testFileA.getAncestors(level=1, type='lfn') == level1, \
            "ERROR: level 1 test failed"
        assert testFileA.getAncestors(level=2, type='lfn') == level2, \
            "ERROR: level 2 test failed"
        assert testFileA.getAncestors(level=3, type='lfn') == level3, \
            "ERROR: level 3 test failed"
        assert testFileA.getAncestors(level=4, type='lfn') == level4, \
            "ERROR: level 4 test failed"
        assert testFileA.getAncestors(level=5, type='lfn') == level5, \
            "ERROR: level 5 test failed"

        assert testFileD.getDescendants(level=1, type='lfn') == level1, \
            "ERROR: level 1 desc test failed"
        assert testFileD.getDescendants(level=2, type='lfn') == decs2, \
            "ERROR: level 2 desc test failed"
        assert testFileD.getDescendants(level=3, type='lfn') == level4, \
            "ERROR: level 3 desc test failed"

        return

    def testGetLocationBulk(self):
        """
        _testGetLocationBulk_

        Checks to see whether the code that we have will enable us to get the locations
        of all files at once
        """
        myThread = threading.currentThread()

        daoFactory = DAOFactory(package="WMCore.WMBS", logger=logging, dbinterface=myThread.dbi)
        locationAction = daoFactory(classname="Locations.New")
        locationAction.execute(siteName="site3", pnn="T2_CH_CERN2")
        locationAction.execute(siteName="site4", pnn="T2_CH_CERN3")
        locationAction.execute(siteName="site5", pnn="T2_CH_CERN4")
        locationAction.execute(siteName="site6", pnn="T2_CH_CERN5")
        locationAction.execute(siteName="site7", pnn="T2_CH_CERN6")

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T2_CH_CERN")
        testFileA.create()

        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T2_CH_CERN2")
        testFileB.create()

        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T2_CH_CERN3")
        testFileC.create()

        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T2_CH_CERN4")
        testFileD.create()

        testFileE = File(lfn="/this/is/a/lfnE", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T2_CH_CERN5")
        testFileE.create()

        testFileF = File(lfn="/this/is/a/lfnF", size=1024, events=10,
                         checksums={'cksum': 1}, locations="T2_CH_CERN6")
        testFileF.create()

        files = [testFileA, testFileB, testFileC, testFileD, testFileE, testFileF]
        # get the file ids only
        ids = [f['id'] for f in files]

        locationFac = daoFactory(classname="Files.GetLocationBulk")
        location = locationFac.execute(files=ids)

        for f in files:
            self.assertEqual(location[f.exists()], list(f['locations']))

        return

    def testBulkParentage(self):
        """
        _testBulkParentage_

        Verify that the bulk parentage dao correctly sets file parentage.
        """
        testFileChildA = File(lfn="/this/is/a/child/lfnA", size=1024,
                              events=20, checksums={'cksum': 1})
        testFileChildB = File(lfn="/this/is/a/child/lfnB", size=1024,
                              events=20, checksums={'cksum': 1})
        testFileChildA.create()
        testFileChildB.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.create()
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileB.create()
        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileC.create()

        parentage = [{"child": testFileChildA["id"], "parent": testFileA["id"]},
                     {"child": testFileChildA["id"], "parent": testFileB["id"]},
                     {"child": testFileChildA["id"], "parent": testFileC["id"]},
                     {"child": testFileChildB["id"], "parent": testFileA["id"]},
                     {"child": testFileChildB["id"], "parent": testFileB["id"]}]

        myThread = threading.currentThread()
        daofactory = DAOFactory(package="WMCore.WMBS",
                                logger=myThread.logger,
                                dbinterface=myThread.dbi)
        bulkParentageAction = daofactory(classname="Files.AddBulkParentage")
        bulkParentageAction.execute(parentage)

        testFileD = File(id=testFileChildA["id"])
        testFileD.loadData(parentage=1)
        testFileE = File(id=testFileChildB["id"])
        testFileE.loadData(parentage=1)

        goldenFiles = [testFileA, testFileB, testFileC]
        for parentFile in testFileD["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"

        goldenFiles = [testFileA, testFileB]
        for parentFile in testFileE["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"
        return

    def testDataStructsFile(self):
        """
        _testDataStructsFile_

        Tests our ability to create a WMBS file from a DataStructs File and vice versa
        """

        myThread = threading.currentThread()

        testLFN = "lfn1"
        testSize = 1024
        testEvents = 100
        testCksum = {"cksum": '1'}
        testParents = set(["lfn2"])
        testRun = Run(1, *[45])
        testSE = "T2_CH_CERN"

        parentFile = File(lfn="lfn2")
        parentFile.create()

        testFile = File()

        inputFile = WMFile(lfn=testLFN, size=testSize, events=testEvents, checksums=testCksum, parents=testParents)
        inputFile.addRun(testRun)
        inputFile.setLocation(pnn=testSE)

        testFile.loadFromDataStructsFile(file=inputFile)
        testFile.create()
        testFile.save()

        loadFile = File(lfn="lfn1")
        loadFile.loadData(parentage=1)

        self.assertEqual(loadFile['size'], testSize)
        self.assertEqual(loadFile['events'], testEvents)
        self.assertEqual(loadFile['checksums'], testCksum)
        self.assertEqual(loadFile['locations'], set([testSE]))
        # self.assertEqual(loadFile['parents'].pop()['lfn'], 'lfn2')

        wmFile = loadFile.returnDataStructsFile()
        self.assertEqual(wmFile == inputFile, True)

        return

    def testParentageByJob(self):
        """
        _testParentageByJob_

        Tests the DAO that assigns parentage by Job
        """

        testWorkflow = Workflow(spec='hello', owner="mnorman",
                                name="wf001", task="basicWorkload/Production")
        testWorkflow.create()
        testFileset = Fileset(name="TestFileset")
        testFileset.create()
        testSubscription = Subscription(fileset=testFileset, workflow=testWorkflow, type="Processing",
                                        split_algo="FileBased")
        testSubscription.create()
        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        testFileParentA = File(lfn="/this/is/a/parent/lfnA", size=1024,
                               events=20, checksums={'cksum': 1},
                               locations=set(['T2_CH_CERN', 'T1_US_FNAL_Disk']))
        testFileParentA.addRun(Run(1, *[45]))
        testFileParentB = File(lfn="/this/is/a/parent/lfnB", size=1024,
                               events=20, checksums={'cksum': 1},
                               locations=set(['T2_CH_CERN', 'T1_US_FNAL_Disk']))
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentA.create()
        testFileParentB.create()
        testFileset.addFile(testFileParentA)
        testFileset.addFile(testFileParentB)
        testFileset.commit()

        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testJobA = Job()
        testJobA["outcome"] = 'success'
        testJobA.create(group=testJobGroup)
        testJobA.addFile(testFileParentA)
        testJobA.addFile(testFileParentB)
        testJobA.associateFiles()
        testSubscription.acquireFiles()
        testJobA.completeInputFiles()

        parentAction = self.daofactory(classname="Files.SetParentageByJob")
        parentAction.execute(binds={'jobid': testJobA.exists(), 'child': testFileA['lfn']})

        testFileB = File(id=testFileA["id"])
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            self.assertEqual(parentFile in goldenFiles, True,
                             "ERROR: Unknown parent file")
            goldenFiles.remove(parentFile)

        self.assertEqual(len(goldenFiles), 0,
                         "ERROR: Some parents are missing")

        testFileC = File(lfn="/this/is/c/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileC.addRun(Run(1, *[46]))
        testFileC.create()

        testJobC = Job()
        testJobC["outcome"] = 'failure'
        testJobC.create(group=testJobGroup)
        testJobC.addFile(testFileParentA)
        testJobC.associateFiles()
        testSubscription.acquireFiles()
        testJobC.failInputFiles()

        parentAction.execute(binds={'jobid': testJobC.exists(), 'child': testFileC['lfn']})

        testFileB = File(id=testFileA["id"])
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            self.assertEqual(parentFile in goldenFiles, True,
                             "ERROR: Unknown parent file")

        testFileC_1 = File(id=testFileC["id"])
        testFileC_1.loadData(parentage=1)

        goldenFiles = [testFileParentA]
        for parentFile in testFileC_1["parents"]:
            self.assertEqual(parentFile in goldenFiles, True,
                             "ERROR: Unknown parent file")

    def testParentageByMergeJob(self):
        """
        _testParentageByJob_

        Tests the DAO that assigns parentage by Job
        """

        testWorkflow = Workflow(spec='hello', owner="mnorman",
                                name="wf001", task="basicWorkload/Production")
        testWorkflow.create()
        testFileset = Fileset(name="TestFileset")
        testFileset.create()
        testSubscription = Subscription(fileset=testFileset, workflow=testWorkflow, type="Processing",
                                        split_algo="FileBased")
        testSubscription.create()
        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        testFileParentA = File(lfn="/this/is/a/parent/lfnA", size=1024,
                               events=20, checksums={'cksum': 1},
                               locations=set(['T2_CH_CERN', 'T1_US_FNAL_Disk']))
        testFileParentA.addRun(Run(1, *[45]))
        testFileParentB = File(lfn="/this/is/a/parent/lfnB", size=1024,
                               events=20, checksums={'cksum': 1},
                               locations=set(['T2_CH_CERN', 'T1_US_FNAL_Disk']))
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentA.create()
        testFileParentB.create()
        testFileset.addFile(testFileParentA)
        testFileset.addFile(testFileParentB)
        testFileset.commit()

        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testJobA = Job()
        testJobA["outcome"] = 'success'
        testJobA.create(group=testJobGroup)
        testJobA.addFile(testFileParentA)
        testJobA.addFile(testFileParentB)
        testJobA.associateFiles()
        testSubscription.acquireFiles()
        testJobA.completeInputFiles()

        parentAction = self.daofactory(classname="Files.SetParentageByMergeJob")
        parentAction.execute(binds={'jobid': testJobA.exists(), 'child': testFileA['lfn']})

        testFileB = File(id=testFileA["id"])
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            self.assertEqual(parentFile in goldenFiles, True,
                             "ERROR: Unknown parent file")
            goldenFiles.remove(parentFile)

        self.assertEqual(len(goldenFiles), 0,
                         "ERROR: Some parents are missing")

        testFileC = File(lfn="/this/is/c/lfn", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileC.addRun(Run(1, *[46]))
        testFileC.create()

        testJobC = Job()
        testJobC["outcome"] = 'failure'
        testJobC.create(group=testJobGroup)
        testJobC.addFile(testFileParentA)
        testJobC.associateFiles()
        testSubscription.acquireFiles()
        testJobC.failInputFiles()

        parentAction.execute(binds={'jobid': testJobC.exists(), 'child': testFileC['lfn']})

        testFileB = File(id=testFileA["id"])
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA]
        for parentFile in testFileB["parents"]:
            self.assertEqual(parentFile in goldenFiles, False,
                             "ERROR: Unknown parent file")

        testFileC_1 = File(id=testFileC["id"])
        testFileC_1.loadData(parentage=1)

        goldenFiles = [testFileParentA]
        for parentFile in testFileC_1["parents"]:
            self.assertEqual(parentFile in goldenFiles, False,
                             "ERROR: Unknown parent file")

    def testAddChecksumsByLFN(self):
        """
        _testAddChecksumsByLFN_

        Tests for adding checksums by DAO by LFN
        """

        testWorkflow = Workflow(spec='hello', owner="mnorman",
                                name="wf001", task="basicWorkload/Production")
        testWorkflow.create()
        testFileset = Fileset(name="TestFileset")
        testFileset.create()
        testSubscription = Subscription(fileset=testFileset, workflow=testWorkflow, type="Processing",
                                        split_algo="FileBased")
        testSubscription.create()
        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10)
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10)
        testFileB.addRun(Run(1, *[45]))
        testFileB.create()

        testJobA = Job()
        testJobA.create(group=testJobGroup)
        testJobA.associateFiles()

        parentAction = self.daofactory(classname="Files.AddChecksumByLFN")
        binds = [{'lfn': testFileA['lfn'], 'cktype': 'cksum', 'cksum': 101},
                 {'lfn': testFileA['lfn'], 'cktype': 'adler32', 'cksum': 201},
                 {'lfn': testFileB['lfn'], 'cktype': 'cksum', 'cksum': 101}]
        parentAction.execute(bulkList=binds)

        testFileC = File(id=testFileA["id"])
        testFileC.loadData()
        testFileD = File(id=testFileB["id"])
        testFileD.loadData()

        self.assertEqual(testFileC['checksums'], {'adler32': '201', 'cksum': '101'})
        self.assertEqual(testFileD['checksums'], {'cksum': '101'})

        return

    def testSetLocationByLFN(self):
        """
        _testSetLocationByLFN_

        Create a file and add a couple locations.  Load the file from the
        database to make sure that the locations were set correctly.
        """
        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10,
                         checksums={'cksum': 1})
        testFileB.addRun(Run(1, *[45]))
        testFileB.create()

        parentAction = self.daofactory(classname="Files.SetLocationByLFN")
        binds = [{'lfn': "/this/is/a/lfnA", 'location': 'T1_US_FNAL_Disk'},
                 {'lfn': "/this/is/a/lfnB", 'location': 'T1_US_FNAL_Disk'}]
        parentAction.execute(lfn=binds)

        testFileC = File(id=testFileA["id"])
        testFileC.loadData()
        testFileD = File(id=testFileB["id"])
        testFileD.loadData()

        self.assertEqual(testFileC['locations'], set(['T1_US_FNAL_Disk']))
        self.assertEqual(testFileD['locations'], set(['T1_US_FNAL_Disk']))

        return

    def testCreateWithParent(self):

        """
        Test passing parnents arguments in file creation.
        check if parent file does not exist, it create the file and set the parentage
        """

        # create parent file before it got added to child file.
        testFileParentA = File(lfn="/this/is/a/parent/lfnA", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentA.addRun(Run(1, *[45]))
        testFileParentA.create()

        # don't create create parent file before it got added to child file.
        testFileParentB = File(lfn="/this/is/a/parent/lfnB", size=1024,
                               events=20, checksums={'cksum': 1})
        testFileParentB.addRun(Run(1, *[45]))

        testFileA = File(lfn="/this/is/a/lfn", size=1024, events=10,
                         checksums={'cksum': 1},
                         parents=[testFileParentA, testFileParentB])
        testFileA.addRun(Run(1, *[45]))

        testFileA.create()

        testFileB = File(id=testFileA["id"])
        testFileB.loadData(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"

    def testAddToFileset(self):
        """
        _AddToFileset_

        Test to see if we can add to a fileset using the DAO
        """
        testFileset = Fileset(name="inputFileset")
        testFileset.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10)
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10)
        testFileB.addRun(Run(1, *[45]))
        testFileB.create()

        addToFileset = self.daofactory(classname="Files.AddToFileset")
        addToFileset.execute(file=[testFileA['lfn'], testFileB['lfn']],
                             fileset=testFileset.id)

        testFileset2 = Fileset(name="inputFileset")
        testFileset2.loadData()

        self.assertEqual(len(testFileset2.files), 2)
        for file in testFileset2.files:
            self.assertTrue(file in [testFileA, testFileB])

        # Check that adding twice doesn't crash
        addToFileset.execute(file=[testFileA['lfn'], testFileB['lfn']],
                             fileset=testFileset.id)

    def testAddDupsToFileset(self):
        """
        _AddToDupsFileset_

        Verify the the dups version of the AddToFileset DAO will not add files
        to a fileset if they're already associated to another fileset with the
        same workflow.
        """
        testWorkflowA = Workflow(spec='hello', owner="mnorman",
                                 name="wf001", task="basicWorkload/Production")
        testWorkflowA.create()
        testWorkflowB = Workflow(spec='hello', owner="mnorman",
                                 name="wf001", task="basicWorkload/Production2")
        testWorkflowB.create()

        testFilesetA = Fileset(name="inputFilesetA")
        testFilesetA.create()
        testFilesetB = Fileset(name="inputFilesetB")
        testFilesetB.create()

        testSubscriptionA = Subscription(workflow=testWorkflowA, fileset=testFilesetA)
        testSubscriptionA.create()
        testSubscriptionB = Subscription(workflow=testWorkflowB, fileset=testFilesetB)
        testSubscriptionB.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10)
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10)
        testFileB.addRun(Run(1, *[45]))
        testFileB.create()

        addToFileset = self.daofactory(classname="Files.AddDupsToFileset")
        addToFileset.execute(file=[testFileA['lfn'], testFileB['lfn']],
                             fileset=testFilesetA.id, workflow="wf001")

        testFileset2 = Fileset(name="inputFilesetA")
        testFileset2.loadData()

        self.assertEqual(len(testFileset2.files), 2)
        for file in testFileset2.files:
            self.assertTrue(file in [testFileA, testFileB])

        # Check that adding twice doesn't crash
        addToFileset.execute(file=[testFileA['lfn'], testFileB['lfn']],
                             fileset=testFilesetA.id, workflow="wf001")

        # Files should not get added to fileset B because fileset A is associated
        # with wf001.
        addToFileset.execute(file=[testFileA['lfn'], testFileB['lfn']],
                             fileset=testFilesetB.id, workflow="wf001")

        testFileset2 = Fileset(name="inputFilesetB")
        testFileset2.loadData()

        self.assertEqual(len(testFileset2.files), 0)
        return

    def testAddDupsToFilesetBulk(self):
        """
        _AddToDupsFilesetBulk_

        Same as testAddDupsToFileset() but faster
        """
        testWorkflowA = Workflow(spec='hello', owner="mnorman",
                                 name="wf001", task="basicWorkload/Production")
        testWorkflowA.create()
        testWorkflowB = Workflow(spec='hello', owner="mnorman",
                                 name="wf001", task="basicWorkload/Production2")
        testWorkflowB.create()

        testFilesetA = Fileset(name="inputFilesetA")
        testFilesetA.create()
        testFilesetB = Fileset(name="inputFilesetB")
        testFilesetB.create()

        testSubscriptionA = Subscription(workflow=testWorkflowA, fileset=testFilesetA)
        testSubscriptionA.create()
        testSubscriptionB = Subscription(workflow=testWorkflowB, fileset=testFilesetB)
        testSubscriptionB.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10, locations=['SiteA'])
        testFileA.addRun(Run(1, *[45]))
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10, locations=['SiteB'])
        testFileB.addRun(Run(1, *[45]))

        addFilesToWMBSInBulk(testFilesetA.id, "wf001",
                             [testFileA, testFileB],
                             conn=testFileA.getDBConn(),
                             transaction=testFileA.existingTransaction())

        testFileset2 = Fileset(name="inputFilesetA")
        testFileset2.loadData()

        self.assertEqual(len(testFileset2.files), 2)
        for file in testFileset2.files:
            self.assertTrue(file in [testFileA, testFileB])

        # Check that adding twice doesn't crash
        addFilesToWMBSInBulk(testFilesetA.id, "wf001",
                             [testFileA, testFileB],
                             conn=testFileA.getDBConn(),
                             transaction=testFileA.existingTransaction())

        # Files should not get added to fileset B because fileset A is associated
        # with wf001.
        addFilesToWMBSInBulk(testFilesetB.id, "wf001",
                             [testFileA, testFileB],
                             conn=testFileA.getDBConn(),
                             transaction=testFileA.existingTransaction())

        testFileset2 = Fileset(name="inputFilesetB")
        testFileset2.loadData()

        self.assertEqual(len(testFileset2.files), 0)
        return

    def test_SetLocationsForWorkQueue(self):
        """
        _SetLocationsForWorkQueue_

        Test the code that sets locations for the WorkQueue
        This is more complicated then it seems.
        """

        action = self.daofactory(classname="Files.SetLocationForWorkQueue")

        testFile = File(lfn="myLFN", size=1024,
                        events=10, checksums={'cksum': 1111})
        testFile.create()

        tFile1 = File(lfn="myLFN")
        tFile1.loadData()
        locations = tFile1.getLocations()

        self.assertEqual(locations, [])

        binds = [{'lfn': 'myLFN', 'location': 'T2_CH_CERN'}]
        action.execute(lfns=['myLFN'], locations=binds)

        tFile1.loadData()
        locations = tFile1.getLocations()
        self.assertEqual(locations, ['T2_CH_CERN'])

        binds = [{'lfn': 'myLFN', 'location': 'T1_US_FNAL_Disk'}]
        action.execute(lfns=['myLFN'], locations=binds)

        tFile1.loadData()
        locations = tFile1.getLocations()
        self.assertEqual(locations, ['T1_US_FNAL_Disk'])

        return


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_ListClosable_

Oracle implementation of Fileset.ListClosable
"""

from WMCore.WMBS.MySQL.Fileset.ListClosable import ListClosable as ListFilesetClosableMySQL

class ListClosable(ListFilesetClosableMySQL):
    pass



#!/usr/bin/env python
"""
_StageOut.Impl_

Implementations of the StageOutImpl for specific protocols

"""
from __future__ import absolute_import


__all__ = []

#  //
# // Each plugin should contain the Registration call at module level
#//  and be imported here, so that all the StageOut plugins can be
#  //imported automatically
# //
#//

from WMCore.Storage.Backends import CPImpl
from WMCore.Storage.Backends import FNALImpl
from WMCore.Storage.Backends import SRMV2Impl
from WMCore.Storage.Backends import LCGImpl
from WMCore.Storage.Backends import XRDCPImpl
from WMCore.Storage.Backends import VandyImpl
from WMCore.Storage.Backends import GFAL2Impl
from WMCore.Storage.Backends import AWSS3Impl
from WMCore.Storage.Backends import UnittestImpl
from WMCore.Storage.Backends import TestFallbackToOldBackend



#!/usr/bin/env python
"""
_GetLocationBulk_

MySQL implementation of File.GetLocationBulk
"""

from WMCore.Database.DBFormatter import DBFormatter


class GetLocationBulk(DBFormatter):
    sql = """SELECT wpnn.pnn as pnn, wfl.fileid as id
               FROM wmbs_file_location wfl
               INNER JOIN wmbs_pnns wpnn ON wfl.pnn = wpnn.id
             WHERE wfl.fileid = :id
            """

    def format(self, rawResults):
        """
        _format_

        Group files into single entries
        """

        results = {}

        for raw in rawResults:
            if raw['id'] not in results:
                results[raw['id']] = []
            results[raw['id']].append(raw['pnn'])

        return results

    def execute(self, files=None, conn=None, transaction=False):
        files = files or []
        if len(files) == 0:
            # Nothing to do
            return

        binds = []
        for fid in files:
            binds.append({'id': fid})

        result = self.dbi.processData(self.sql, binds,
                                      conn=conn, transaction=transaction)

        return self.format(self.formatDict(result))



#!/usr/bin/env python

from __future__  import division

import os
import time
import shutil
import random
import os.path
import getpass
import unittest
import threading

from subprocess import Popen, PIPE
import pickle

# Imports for testing
from WMQuality.TestInit import TestInit
from WMQuality.Emulators import EmulatorSetup
from WMCore.DAOFactory import DAOFactory
from WMCore.WMInit import getWMBASE



# Component imports
from WMComponent.JobCreator.JobCreatorPoller       import JobCreatorPoller
from WMComponent.JobSubmitter.JobSubmitterPoller   import JobSubmitterPoller
from WMComponent.JobTracker.JobTrackerPoller       import JobTrackerPoller
from WMComponent.JobAccountant.JobAccountantPoller import JobAccountantPoller
from WMComponent.JobArchiver.JobArchiverPoller     import JobArchiverPoller
from WMComponent.TaskArchiver.TaskArchiverPoller   import TaskArchiverPoller
from WMComponent.RetryManager.RetryManagerPoller   import RetryManagerPoller
from WMComponent.ErrorHandler.ErrorHandlerPoller   import ErrorHandlerPoller


# WMBS Objects
from WMCore.WMBS.Job          import Job
from WMCore.WMBS.File         import File
from WMCore.WMBS.Fileset      import Fileset
from WMCore.WMBS.Workflow     import Workflow
from WMCore.WMBS.Subscription import Subscription

# Agent imports
from WMCore.Agent.HeartbeatAPI              import HeartbeatAPI
from WMCore.Agent.Configuration             import Configuration

# WMCore library imports
from WMCore.ResourceControl.ResourceControl  import ResourceControl
from WMCore.FwkJobReport.Report              import Report

# WMSpec stuff
from WMCore.WMSpec.Makers.TaskMaker import TaskMaker
from WMCore_t.WMSpec_t.TestSpec     import testWorkload



def getCondorRunningJobs(user = None):
    """
    _getCondorRunningJobs_

    Return the number of jobs currently running for a user
    """
    if not user:
        user = getpass.getuser()

    command = ['condor_q', user]
    pipe = Popen(command, stdout = PIPE, stderr = PIPE, shell = False)
    stdout, error = pipe.communicate()

    output = stdout.split('\n')[-2]

    nJobs = int(output.split(';')[0].split()[0])

    return nJobs

def condorRM(user = None):
    # Now clean-up
    if not user:
        user = getpass.getuser()
    command = ['condor_rm', user]
    pipe = Popen(command, stdout = PIPE, stderr = PIPE, shell = False)
    pipe.communicate()
    return


class WMAgentTest(unittest.TestCase):
    """
    _WMAgentTest_

    Global unittest for all WMAgent components
    """

    # This is an integration test
    __integration__ = "Any old bollocks"

    sites = ['T2_US_Florida', 'T2_US_UCSD', 'T2_TW_Taiwan', 'T1_CH_CERN']
    components = ['JobCreator', 'JobSubmitter', 'JobTracker',
                  'JobAccountant', 'JobArchiver', 'TaskArchiver',
                  'RetryManager', 'ErrorHandler']

    def setUp(self):
        """
        _setUp_

        Set up vital components
        """

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules = ["WMCore.WMBS",'WMCore.MsgService',
                                                 'WMCore.ResourceControl', 'WMCore.ThreadPool',
                                                 'WMCore.Agent.Database'],
                                useDefault = False)

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package = "WMCore.WMBS",
                                     logger = myThread.logger,
                                     dbinterface = myThread.dbi)



        locationAction = self.daoFactory(classname = "Locations.New")
        pendingSlots  = self.daoFactory(classname = "Locations.SetPendingSlots")


        for site in self.sites:
            locationAction.execute(siteName = site, pnn = 'se.%s' % (site), ceName = site)
            pendingSlots.execute(siteName = site, pendingSlots = 1000)


        #Create sites in resourceControl
        resourceControl = ResourceControl()
        for site in self.sites:
            resourceControl.insertSite(siteName = site, pnn = 'se.%s' % (site), ceName = site)
            resourceControl.insertThreshold(siteName = site, taskType = 'Processing', \
                                            maxSlots = 10000, pendingSlots = 10000)


        self.testDir = self.testInit.generateWorkDir()


        # Set heartbeat
        for component in self.components:
            heartbeatAPI = HeartbeatAPI(component)
            heartbeatAPI.registerComponent()

        self.configFile = EmulatorSetup.setupWMAgentConfig()

        return


    def tearDown(self):
        """
        _tearDown_

        Tear down everything and go home.
        """

        self.testInit.clearDatabase()

        self.testInit.delWorkDir()

        EmulatorSetup.deleteConfig(self.configFile)

        return




    def createTestWorkload(self, workloadName = 'Test', emulator = True):
        """
        _createTestWorkload_

        Creates a test workload for us to run on, hold the basic necessities.
        """


        workload = testWorkload("TestWorkload")
        rereco = workload.getTask("ReReco")


        taskMaker = TaskMaker(workload, os.path.join(self.testDir, 'workloadTest'))
        taskMaker.skipSubscription = True
        taskMaker.processWorkload()

        workload.save(workloadName)

        return workload




    def getConfig(self):
        """
        _getConfig_

        This is the global test configuration object
        """



        config = self.testInit.getConfiguration()

        config.component_("Agent")
        config.Agent.WMSpecDirectory = self.testDir
        config.Agent.agentName       = 'testAgent'
        config.Agent.componentName   = 'test'


        # First the general stuff
        config.section_("General")
        config.General.workDir = os.getenv("TESTDIR", self.testDir)

        # Now the CoreDatabase information
        # This should be the dialect, dburl, etc

        config.section_("CoreDatabase")
        config.CoreDatabase.connectUrl = os.getenv("DATABASE")
        config.CoreDatabase.socket     = os.getenv("DBSOCK")



        # JobCreator
        config.component_("JobCreator")
        config.JobCreator.namespace = 'WMComponent.JobCreator.JobCreator'
        config.JobCreator.logLevel  = 'DEBUG'
        config.JobCreator.maxThreads                = 1
        config.JobCreator.UpdateFromResourceControl = True
        config.JobCreator.pollInterval              = 10
        config.JobCreator.jobCacheDir               = self.testDir
        config.JobCreator.defaultJobType            = 'processing' #Type of jobs that we run, used for resource control
        config.JobCreator.workerThreads             = 2
        config.JobCreator.componentDir              = os.path.join(os.getcwd(), 'Components')



        # JobSubmitter
        config.component_("JobSubmitter")
        config.JobSubmitter.namespace     = 'WMComponent.JobSubmitter.JobSubmitter'
        config.JobSubmitter.logLevel      = 'INFO'
        config.JobSubmitter.maxThreads    = 1
        config.JobSubmitter.pollInterval  = 10
        config.JobSubmitter.pluginName    = 'CondorGlobusPlugin'
        config.JobSubmitter.pluginDir     = 'JobSubmitter.Plugins'
        config.JobSubmitter.submitDir     = os.path.join(self.testDir, 'submit')
        config.JobSubmitter.submitNode    = os.getenv("HOSTNAME", 'badtest.fnal.gov')
        config.JobSubmitter.submitScript  = os.path.join(getWMBASE(),
                                                         'test/python/WMComponent_t/JobSubmitter_t',
                                                         'submit.sh')
        config.JobSubmitter.componentDir  = os.path.join(os.getcwd(), 'Components')
        config.JobSubmitter.workerThreads = 2
        config.JobSubmitter.jobsPerWorker = 200




        # JobTracker
        config.component_("JobTracker")
        config.JobTracker.logLevel      = 'DEBUG'
        config.JobTracker.pollInterval  = 10
        config.JobTracker.trackerName   = 'CondorTracker'
        config.JobTracker.pluginDir     = 'WMComponent.JobTracker.Plugins'
        config.JobTracker.componentDir  = os.path.join(os.getcwd(), 'Components')
        config.JobTracker.runTimeLimit  = 7776000 #Jobs expire after 90 days
        config.JobTracker.idleTimeLimit = 7776000
        config.JobTracker.heldTimeLimit = 7776000
        config.JobTracker.unknTimeLimit = 7776000



        # JobAccountant
        config.component_("JobAccountant")
        config.JobAccountant.pollInterval = 60
        config.JobAccountant.componentDir = os.path.join(os.getcwd(), 'Components')
        config.JobAccountant.logLevel     = 'INFO'



        # JobArchiver
        config.component_("JobArchiver")
        config.JobArchiver.pollInterval          = 60
        config.JobArchiver.logLevel              = 'INFO'
        config.JobArchiver.logDir                = os.path.join(self.testDir, 'logs')
        config.JobArchiver.componentDir          = os.path.join(os.getcwd(), 'Components')
        config.JobArchiver.numberOfJobsToCluster = 1000



        # Task Archiver
        config.component_("TaskArchiver")
        config.TaskArchiver.componentDir    = self.testInit.generateWorkDir()
        config.TaskArchiver.WorkQueueParams = {}
        config.TaskArchiver.pollInterval    = 60
        config.TaskArchiver.logLevel        = 'INFO'
        config.TaskArchiver.timeOut         = 0



        # JobStateMachine
        config.component_('JobStateMachine')
        config.JobStateMachine.couchurl        = os.getenv('COUCHURL',
                                                           'mnorman:theworst@cmssrv52.fnal.gov:5984')
        config.JobStateMachine.couchDBName     = "mnorman_test"


        # Needed, because this is a test
        os.makedirs(config.JobSubmitter.submitDir)


        return config



    def createFileCollection(self, name, nSubs, nFiles, workflowURL = 'test', site = None):
        """
        _createFileCollection_

        Create a collection of files for splitting into jobs
        """

        myThread = threading.currentThread()

        testWorkflow = Workflow(spec = workflowURL, owner = "mnorman",
                                name = name, task="/TestWorkload/ReReco")
        testWorkflow.create()

        for sub in range(nSubs):

            nameStr = '%s-%i' % (name, sub)

            testFileset = Fileset(name = nameStr)
            testFileset.create()

            for f in range(nFiles):
                # pick a random site
                if not site:
                    tmpSite = 'se.%s' % (random.choice(self.sites))
                else:
                    tmpSite = 'se.%s' % (site)
                testFile = File(lfn = "/lfn/%s/%i" % (nameStr, f), size = 1024, events = 10)
                testFile.setLocation(tmpSite)
                testFile.create()
                testFileset.addFile(testFile)

            testFileset.commit()
            testFileset.markOpen(isOpen = 0)
            testSubscription = Subscription(fileset = testFileset,
                                            workflow = testWorkflow,
                                            type = "Processing",
                                            split_algo = "FileBased")
            testSubscription.create()


        return



    def createReports(self, jobs, retryCount = 0):
        """
        _createReports_

        Create some dummy job reports for each job
        """


        report = Report()
        report.addStep('testStep', 0)

        for job in jobs:
            #reportPath = os.path.join(job['cache_dir'], 'Report.%i.pkl' % (retryCount))
            reportPath = job['fwjr_path']
            if os.path.exists(reportPath):
                os.remove(reportPath)
            report.save(reportPath)


        return



    def testA_StraightThrough(self):
        """
        _StraightThrough_

        Just run everything straight through without any variations
        """
        # Do pre-submit job check
        nRunning = getCondorRunningJobs()
        self.assertEqual(nRunning, 0, "User currently has %i running jobs.  Test will not continue" % (nRunning))

        myThread = threading.currentThread()
        workload = self.createTestWorkload()
        config   = self.getConfig()


        name         = 'WMAgent_Test1'
        site         = self.sites[0]
        nSubs        = 5
        nFiles       = 10
        workloadPath = os.path.join(self.testDir, 'workloadTest',
                                    'TestWorkload', 'WMSandbox',
                                    'WMWorkload.pkl')

        # Create a collection of files
        self.createFileCollection(name = name, nSubs = nSubs,
                                  nFiles = nFiles,
                                  workflowURL = workloadPath,
                                  site = site)



        ############################################################
        # Test the JobCreator


        config.Agent.componentName = 'JobCreator'
        testJobCreator = JobCreatorPoller(config = config)

        testJobCreator.algorithm()
        time.sleep(5)


        # Did all jobs get created?
        getJobsAction = self.daoFactory(classname = "Jobs.GetAllJobs")
        result = getJobsAction.execute(state = 'Created', jobType = "Processing")
        self.assertEqual(len(result), nSubs*nFiles)


        # Count database objects
        result = myThread.dbi.processData('SELECT * FROM wmbs_sub_files_acquired')[0].fetchall()
        self.assertEqual(len(result), nSubs * nFiles)


        # Find the test directory
        testDirectory = os.path.join(self.testDir, 'TestWorkload', 'ReReco')
        self.assertTrue('JobCollection_1_0' in os.listdir(testDirectory))
        self.assertTrue(len(os.listdir(testDirectory)) <= 20)

        groupDirectory = os.path.join(testDirectory, 'JobCollection_1_0')

        # First job should be in here
        self.assertTrue('job_1' in os.listdir(groupDirectory))
        jobFile = os.path.join(groupDirectory, 'job_1', 'job.pkl')
        self.assertTrue(os.path.isfile(jobFile))
        with open(jobFile, 'rb') as f:
            job = pickle.load(f)


        self.assertEqual(job['workflow'], name)
        self.assertEqual(len(job['input_files']), 1)
        self.assertEqual(os.path.basename(job['sandbox']), 'TestWorkload-Sandbox.tar.bz2')










        ###############################################################
        # Now test the JobSubmitter

        config.Agent.componentName = 'JobSubmitter'
        testJobSubmitter = JobSubmitterPoller(config = config)


        testJobSubmitter.algorithm()


        # Check that jobs are in the right state
        result = getJobsAction.execute(state = 'Created', jobType = "Processing")
        self.assertEqual(len(result), 0)
        result = getJobsAction.execute(state = 'Executing', jobType = "Processing")
        self.assertEqual(len(result), nSubs * nFiles)



        # Check assigned locations
        getLocationAction = self.daoFactory(classname = "Jobs.GetLocation")
        for id in result:
            loc = getLocationAction.execute(jobid = id)
            self.assertEqual(loc, [[site]])


        # Check to make sure we have running jobs
        nRunning = getCondorRunningJobs()
        self.assertEqual(nRunning, nFiles * nSubs)


        #################################################################
        # Now the JobTracker


        config.Agent.componentName = 'JobTracker'
        testJobTracker = JobTrackerPoller(config = config)
        testJobTracker.setup()

        testJobTracker.algorithm()

        # Running the algo without removing the jobs should do nothing
        result = getJobsAction.execute(state = 'Executing', jobType = "Processing")
        self.assertEqual(len(result), nSubs * nFiles)


        condorRM()
        time.sleep(1)

        # All jobs gone?
        nRunning = getCondorRunningJobs()
        self.assertEqual(nRunning, 0)


        testJobTracker.algorithm()
        time.sleep(5)

        # Running the algo without removing the jobs should do nothing
        result = getJobsAction.execute(state = 'Executing', jobType = "Processing")
        self.assertEqual(len(result), 0)
        result = getJobsAction.execute(state = 'Complete', jobType = "Processing")
        self.assertEqual(len(result), nSubs * nFiles)




        #################################################################
        # Now the JobAccountant

        # First you need to load all jobs


        self.getFWJRAction = self.daoFactory(classname = "Jobs.GetFWJRByState")
        completeJobs       = self.getFWJRAction.execute(state = "complete")


        # Create reports for all jobs
        self.createReports(jobs = completeJobs, retryCount = 0)






        config.Agent.componentName = 'JobAccountant'
        testJobAccountant = JobAccountantPoller(config = config)
        testJobAccountant.setup()


        # It should do something with the jobs
        testJobAccountant.algorithm()


        # All the jobs should be done now
        result = getJobsAction.execute(state = 'Complete', jobType = "Processing")
        self.assertEqual(len(result), 0)
        result = getJobsAction.execute(state = 'Success', jobType = "Processing")
        self.assertEqual(len(result), nSubs * nFiles)



        #######################################################################
        # Now the JobArchiver


        config.Agent.componentName = 'JobArchiver'
        testJobArchiver = JobArchiverPoller(config = config)


        testJobArchiver.algorithm()

        # All the jobs should be cleaned up
        result = getJobsAction.execute(state = 'Success', jobType = "Processing")
        self.assertEqual(len(result), 0)
        result = getJobsAction.execute(state = 'Cleanout', jobType = "Processing")
        self.assertEqual(len(result), nSubs * nFiles)


        logDir = os.path.join(self.testDir, 'logs')

        for job in completeJobs:
            self.assertFalse(os.path.exists(job['fwjr_path']))
            jobFolder = 'JobCluster_%i' \
                    % (int(job['id']/config.JobArchiver.numberOfJobsToCluster))
            jobPath = os.path.join(logDir, jobFolder, 'Job_%i.tar' %(job['id']))
            self.assertTrue(os.path.isfile(jobPath))
            self.assertTrue(os.path.getsize(jobPath) > 0)




        ###########################################################################
        # Now the TaskAchiver


        config.Agent.componentName = 'TaskArchiver'
        testTaskArchiver = TaskArchiverPoller(config = config)


        testTaskArchiver.algorithm()


        result = getJobsAction.execute(state = 'Cleanout', jobType = "Processing")
        self.assertEqual(len(result), 0)


        for jdict in completeJobs:
            job = Job(id = jdict['id'])
            self.assertFalse(job.exists())





        if os.path.isdir('testDir'):
            shutil.rmtree('testDir')
        shutil.copytree('%s' %self.testDir, os.path.join(os.getcwd(), 'testDir'))




        return







if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
Test that the following dburl's are correctly made:

            postgres://username:password@host:port/database

            mysql://host/database
            mysql://username@host/database
            mysql://username:password@host:port/database

            oracle://username:password@tnsName
            oracle://username:password@host:port/sidname
"""
from __future__ import print_function





import unittest
import logging

from WMCore.Database.DBFactory import DBFactory

class DBFactoryTest(unittest.TestCase):
    def setUp(self):
        "make a logger instance and create tables"

        logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                    datefmt='%m-%d %H:%M',
                    filename='%s.log' % __file__.replace('.py',''),
                    filemode='w')

        self.logger = logging.getLogger('DBFactoryTest')

    def urlTest(self, url, testurl):
        self.asserEqual(url ,testurl, '%s is incorrect should be %s' % (url, testurl))
        return True

    def genericTest(self, testname=None, testoptions={}, testurl=None):
        try:
            dbf = DBFactory(self.logger, options=testoptions)
            if self.urlTest(dbf.dburl, testurl):
                print(testname, " : ", testurl, "  .... OK!")
            else:
                print(testname, " : ", testurl, "  .... FAIL!")
        except Exception as e:
            print(testname, " : ", testurl, "  .... EXCEPTION!", e)

    def testAllURLs(self):
        """
        Checks all the url's we use in the tests are valid
        """

        urls = [
                'postgres://username:password@host:1234/database',
                'mysql://host/database',
                'mysql://username@host/database',
                'mysql://username:password@host:1234/database',
                'oracle://username:password@tnsName',
                'oracle://username:password@host:1234/sidname'
                ]
        for testurl in urls:
            try:
                dbf = DBFactory(self.logger, dburl=testurl)
                if self.urlTest(dbf.dburl, testurl):
                    print("testAllURLs : ", testurl, "  .... OK!")
                else:
                    print("testAllURLs : ", testurl, "  .... FAIL!")
            except Exception as e:
                print("testAllURLs : ", testurl, "  .... EXCEPTION!", e)

    def testPostGres(self):
        """
        Test that factory correctly makes:
            postgres://username:password@host:1234/postgresdb
        """

        self.genericTest(testname='testPostGres',
                         testoptions={'dialect':'postgres',
                                      'database':'database',
                                      'username':'username',
                                      'password':'password',
                                      'host':'host',
                                      'port':1234},
                         testurl='postgres://username:password@host:1234/database')

    def testMySQL(self):
        """
        Test that factory correctly makes:
            mysql://host/database
            mysql://username@host/database
            mysql://username:password@host:1234/database
        """

        self.genericTest(testname='testMySQL',
                         testoptions={'dialect':'mysql',
                                      'host':'host',
                                      'database':'database'},
                         testurl='mysql://host/database')

        self.genericTest(testname='testMySQL',
                         testoptions={'dialect':'mysql',
                                      'host':'host',
                                      'username':'username',
                                      'database':'database'},
                         testurl='mysql://username@host/database')

        self.genericTest(testname='testMySQL',
                         testoptions={'dialect':'mysql',
                                      'host':'host',
                                      'username':'username',
                                      'password':'password',
                                      'database':'database',
                                      'port':1234},
                         testurl='mysql://username:password@host:1234/database')

    def testOracle(self):
        """
        Test that factory correctly makes:
            oracle://username:password@tnsName
            oracle://username:password@host:1234/sidname
        """

        self.genericTest(testname='testOracle',
                         testoptions={'dialect':'oracle',
                                      'sidname':'sidname',
                                      'username':'username',
                                      'password':'password',
                                      'tnsName':'tnsName'},
                         testurl='oracle://username:password@tnsName')

        self.genericTest(testname='testOracle',
                         testoptions={'dialect':'oracle',
                                      'sid':'sidname',
                                      'username':'username',
                                      'password':'password',
                                      'host':'host',
                                      'port':1234},
                         testurl='oracle://username:password@host:1234/sidname')

        self.genericTest(testname='testOracle',
                         testoptions={'dialect':'oracle',
                                      'database':'sidname',
                                      'username':'username',
                                      'password':'password',
                                      'host':'host',
                                      'port':1234},
                         testurl='oracle://username:password@host:1234/sidname')




if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_LoadFiles_

Oracle implementation of Jobs.LoadFiles
"""

__all__ = []



from WMCore.Database.DBFormatter import DBFormatter
from WMCore.WMBS.MySQL.Jobs.LoadFiles import LoadFiles as LoadFilesMySQL

class LoadFiles(LoadFilesMySQL):
    """
    _LoadFiles_

    Retrieve all files that are associated with the given job from the
    database.
    """
    sql = "SELECT fileid FROM wmbs_job_assoc WHERE JOB = :jobid"

    def formatDict(self, results):
        """
        _formatDict_

        Cast the file attribute to an integer,.
        """
        formattedResults = DBFormatter.formatDict(self, results)

        dictResults = []
        for formattedResult in formattedResults:
            dictResult = {}
            if "fileid" in formattedResult:
                dictResult["id"] = int(formattedResult["fileid"])
                dictResults.append(dictResult)

        return dictResults



import re

# path to static resources
RX_STATIC_DIR_PATH = re.compile(r"^([a-zA-Z]+/)+[-a-z0-9_]+\.(?:css|js|png|gif|html)$")

# name of the request in the ReqMgr
RX_REQUEST_NAME = re.compile(r"^[-A-Za-z0-9_]+$")

# name of a group in the ReqMgr
RX_GROUP_NAME = re.compile(r"^[-A-Za-z0-9_]+$")

# name of a team in the ReqMgr
RX_TEAM_NAME = re.compile(r"^[-A-Za-z0-9_]+$")

# boolean flag in the URL argument
RX_BOOL_FLAG = re.compile(r"^(true|false)$")



#!/usr/bin/env python
"""
_BulkAddByLFN_

Oracle implementation of Fileset.BulkAddByLFN
"""

from WMCore.WMBS.MySQL.Fileset.BulkAddByLFN import BulkAddByLFN as MySQLBulkAddByLFN

class BulkAddByLFN(MySQLBulkAddByLFN):
    sql = """INSERT INTO wmbs_fileset_files (fileid, fileset, insert_time)
               SELECT (SELECT id FROM wmbs_file_details WHERE lfn = :lfn), :fileset, :timestamp FROM DUAL"""

    sqlAvail = """INSERT INTO wmbs_sub_files_available (subscription, fileid)
                    SELECT wmbs_subscription.id AS subscription,
                           wmbs_file_details.id AS fileid FROM wmbs_subscription
                      INNER JOIN wmbs_file_details ON
                        wmbs_file_details.lfn = :lfn
                    WHERE wmbs_subscription.fileset = :fileset"""



#!/usr/bin/env python
"""
Unittests for IteratorTools functions
"""

from __future__ import division, print_function

from builtins import range
import itertools
import unittest

from Utils.IteratorTools import grouper, flattenList


class IteratorToolsTest(unittest.TestCase):
    """
    unittest for IteratorTools functions
    """

    def testGrouper(self):
        """
        Test the grouper function (returns chunk of an iterable)
        """

        listChunks = [i for i in grouper(list(range(0, 7)), 3)]  # Want list(range) for python 3
        iterChunks = [i for i in grouper(range(0, 7), 3)]  # xrange becomes range in python 3

        for a, b in itertools.zip_longest(listChunks, iterChunks):
            self.assertEqual(a, b)

        self.assertEqual(listChunks[-1], [6])

    def testFlattenList(self):
        """
        Test the flattenList function (returns a flat list out
        of a list of lists)
        """
        doubleList = [list(range(1, 4)), list(range(10, 11)), list(range(15, 18))]
        flatList = flattenList(doubleList)
        self.assertEqual(len(flatList), 7)
        self.assertEqual(set(flatList), set([1, 2, 3, 10, 15, 16, 17]))


if __name__ == '__main__':
    unittest.main()
